 unit MedLog;
{
================================================================================
*	File:  MedLog.PAS
*
*	Application:  Bar Code Medication Administration
*	Revision:     $Revision: 25 $  $Modtime: 2/06/02 12:29p $
*
*	Description:  This form is for logging transactions for a patient's active
*               medication orders.  There is a tabsheet for each type of
*               transaction.  Only the tabsheet for one transaction type is
*               visible at a time.
*
*	Notes:
*
================================================================================
}

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	ComCtrls, StdCtrls, Buttons, ExtCtrls,
	BCMA_Objects;

type
	TfrmMedLog = class(TForm)
	(*
			A form is for logging transactions for a patient's active medication
			orders.  There is a tabsheet for each type of transaction.  Only the
			tabsheet for one transaction type is visible at a time.
	*)

		PageControl: TPageControl;
		tsConfirmPRN: TTabSheet;
		tsPRNEffectiveness: TTabSheet;
		tsConfirmContinuous: TTabSheet;
		tsAddComment: TTabSheet;
		tsNotGiven: TTabSheet;
		btnSubmit: TBitBtn;
		BitBtn2: TBitBtn;
		lbxReasons: TListBox;
		Panel1: TPanel;
		lblActiveMedication: TLabel;
		lblDispensedDrug: TLabel;
		Label3: TLabel;
		mmoSpecialInstructions: TMemo;
		mmoMessage: TMemo;
    lblMessage: TLabel;
		mmoAddComment: TMemo;
		Panel2: TPanel;
		Label2: TLabel;
		Label4: TLabel;
		Label5: TLabel;
		Label6: TLabel;
		Label7: TLabel;
		Label1: TLabel;
    Label8: TLabel;
		Label9: TLabel;
    TabSheet1: TTabSheet;
    btnOK: TButton;
    btnCancel: TButton;
		procedure FormCreate(Sender: TObject);
		(*
				Initially sets the visibility of all tabs to False.  Clears all display
				areas.
		*)

		procedure FormShow(Sender: TObject);
		(*
				Checks the current MedOrder for validity, then fills readonly fields.
				Sets up the active page corresponding to the transaction type.
		*)

		procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
		(*
				Checks to make sure all required data is entered before closing the
				form.  Uses method CanCloseCheck prior to allowing an mrOK close.
		*)

		procedure btnOKClick(Sender: TObject);
		(*
				Sets ModalResult to mrOK, closing the form.
		*)

		procedure lbxReasonsEnter(Sender: TObject);
		(*
				Enables the Submit button, since a reason has now been selected.
		*)

		procedure lbxReasonsKeyPress(Sender: TObject; var Key: Char);
		(*
				If the <Enter> key is pressed, the btnOKClick method is run.
		*)

    procedure lbxReasonsClick(Sender: TObject);
    {
      if an item is selected, then enable the ok button
    }

    procedure mmoAddCommentChange(Sender: TObject);
    {
      If a comment is added, enable the ok button.
    }


	private
		{ Private declarations }
		function CanCloseCheck: boolean;
		(*
				Returns True if all required data has been entered or selected.
		*)

	public
		{ Public declarations }
		MedOrder:	TBCMA_MedOrder;
		MedLogType: TMedLogTypes;
	end;

var
	frmMedLog: TfrmMedLog;

implementation
{$R *.DFM}

uses
	BCMA_Common, MFunStr;

procedure TfrmMedLog.lbxReasonsEnter(Sender: TObject);
begin
	btnSubmit.Enabled := True;
end;

procedure TfrmMedLog.FormCreate(Sender: TObject);
var
	ii: integer;
begin
	with PageControl do
		for ii := 0 to PageCount-1 do
			pages[ii].TabVisible := False;

	mmoMessage.clear;
	mmoAddComment.clear;
	lbxReasons.items.clear;
end;

procedure TfrmMedLog.FormShow(Sender: TObject);
begin
//	if MedOrder = nil then
//		messageDlg('Cannot Add a Comment:  No Medication Order Found!', mtError, [mbOK], 0)
//	else
//		begin
//			with MedOrder do
//				if ValidOrder or ((OrderStatus = 'H') and (not ValidOrder)) then
					//begin
//						lblActiveMedication.caption := lblActiveMedication.caption + ActiveMedication;
//						if OrderedDrugNames.count > 0 then
//							lblDispensedDrug.caption := lblDispensedDrug.caption + OrderedDrugNames[0]
//						else
//							lblDispensedDrug.caption := '';
//						mmoSpecialInstructions.text := SpecialInstructions;
//						mmoMessage.text := StatusMessage;

						with PageControl do
							begin
								ActivePage.TabVisible := True;

								if 	(ActivePage = tsPRNEffectiveness) or
										(ActivePage = tsConfirmContinuous) or
										(ActivePage = tsAddComment) then
									with mmoAddComment do
										begin
											parent := ActivePage;
											clear;
											show;
											setFocus;
										end;

								if 	(ActivePage = tsNotGiven) then
									with lbxReasons do
										begin
											parent := ActivePage;
											itemIndex := -1;
											show;
											setFocus;
                    end;

                if 	(ActivePage = tsConfirmPRN) then
                  with lbxReasons do
                    begin
                      parent := ActivePage;
											lblMessage.Caption := 'Last Four Actions:';
                      itemIndex := -1;
                      show;
                      setFocus;
                    end;
              end;
          //end;
    //end;
end;

procedure TfrmMedLog.btnOKClick(Sender: TObject);
begin
//	modalResult := mrOK;
end;

procedure TfrmMedLog.lbxReasonsKeyPress(Sender: TObject; var Key: Char);
begin
	if key = chr(VK_RETURN) then
		btnOKClick(Sender);
end;

function TfrmMedLog.CanCloseCheck: boolean;
var
	errMsg:		string;
begin
	result := False;
			with PageControl do
				begin
					if 	(ActivePage = tsPRNEffectiveness) or
							(ActivePage = tsConfirmContinuous) or
							(ActivePage = tsAddComment) then
						if mmoAddComment.text <> '' then
							begin
								cmtUserComments := mmoAddComment.text;
								result := True;
							end
						else
							errMsg := 'You Must Enter a Comment!';

					if 	(ActivePage = tsConfirmPRN) then
						with lbxReasons do
							if itemIndex > -1 then
								begin
									cmtReasonGivenPRN := Items[lbxReasons.itemIndex];
									result := True;
								end
							else
								errMsg := 'You Must Select a Reason from the List!';

					if 	(ActivePage = tsNotGiven) then
						with lbxReasons do
							if itemIndex > -1 then
								begin
									cmtUserComments := Items[lbxReasons.itemIndex];
									result := True;
								end
							else
								errMsg := 'You Must Select a Reason from the List!';
				end;
		//end;
end;

procedure TfrmMedLog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
	canClose := (modalResult = mrCancel) or CanCloseCheck;
  if not CanClose then
    PageControl.ActivePage.setFocus;
end;

procedure TfrmMedLog.lbxReasonsClick(Sender: TObject);
begin
  if lbxReasons.ItemIndex <> -1 then btnOK.Enabled := True;
end;

procedure TfrmMedLog.mmoAddCommentChange(Sender: TObject);
begin
  btnOK.Enabled := True;
end;

end.
