  Unit MultOrder;
{
================================================================================
*	File:  MultOrder.PAS
*
*	Application:  Bar Code Medication Administration
*	Revision:     $Revision: 20 $  $Modtime: 5/06/02 10:20a $
*
*	Description:  This is the form for displaying multiple orders for a given
*								scanned dispensed drug for the application.  The form will
*               resize itself at creation time, according to the number of
*               lines and the length of the longest line in the listbox.
*
*	Notes:
*
*
================================================================================
}

Interface

Uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons, Grids, ComCtrls, BCMA_Util;

Const
  MAXCHARS = 110;
  MINCHARS = 30;
  MAXLINES = 40;
  MINLINES = 5;

Type
  TfrmMultOrder = Class(TForm)
    Panel1: TPanel;
    lblDispensedDrug: TLabel;
    Label1: TLabel;
    pnlButton: TPanel;
    hdrMultiOrder: THeaderControl;
    btnCancel: TButton;
    btnOK: TButton;
    lstMultiOrder: TListBox;
    Procedure FormShow(Sender: TObject);
  (*
    Sets the initial size of the form according to the number of lines and
    the length of the longest line of text in the report.
  *)

    Procedure btnOKClick(Sender: TObject);
  (*
    If a list item has been selected, modalResult is set to ItemIndex + 100,
    thus closing the form.
  *)

    Procedure pnlButtonResize(Sender: TObject);
  (*
    Resets the positions of the OK and Cancel buttons whenever the form
    is resized, to ensure that they will always be visible.
  *)

    Procedure lbxScannedOrdersEnter(Sender: TObject);
    {
      Enables the OK button, since an item has now been selected from the list.
    }

    procedure sgMultOrderEnter(Sender: TObject);
    {
    }

    procedure lstMultiOrderMeasureItem(Control: TWinControl;
      Index: Integer; var Height: Integer);
    {
      Called to calculate the height of each row
    }

    procedure lstMultiOrderDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    {
      Draws the data on the canvas including the gridlines
    }

    procedure hdrMultiOrderSectionResize(HeaderControl: THeaderControl;
      Section: THeaderSection);
    {
      When a headersection is resized, recalculate the max size of each headersection,
      and re-paint lstMultiOrder with the new column sizes.
    }

    procedure lstMultiOrderClick(Sender: TObject);
    {
      if an item is selected, enable the OK button.
    }



  Private
  { Private declarations }
  Public
  { Public declarations }
  End;

Var
  frmMultOrder: TfrmMultOrder;
  ItemsHeight: Integer;

Implementation

{$R *.DFM}

Uses
  Math, BCMA_Common, BCMA_Objects, BCMA_Main;

Procedure TfrmMultOrder.FormShow(Sender: TObject);
Begin
  with self do
    begin
      width := frmmain.Width - 20;
      height := frmmain.Height div 2;
    end;
  if tag = 0 then
    btnCancel.setFocus

  else
    begin
      btnCancel.Visible := false;
      btnOk.Enabled := True;
      btnOk.Left := btnCancel.Left;
    end;
End;

Procedure TfrmMultOrder.btnOKClick(Sender: TObject);
Begin
  modalResult := lstMultiOrder.ItemIndex + 100;
End;

Procedure TfrmMultOrder.pnlButtonResize(Sender: TObject);
Begin
  btnOK.Width := btnCancel.Width;
  btnOK.Left := pnlButton.ClientWidth - (btnOK.Width + self.BorderWidth + 5) * 2;
  btnCancel.Left := pnlButton.ClientWidth - (btnCancel.Width + self.BorderWidth + 5);
End;

Procedure TfrmMultOrder.lbxScannedOrdersEnter(Sender: TObject);
Begin
  btnOK.enabled := True;
End;

procedure TfrmMultOrder.sgMultOrderEnter(Sender: TObject);
begin
  btnOK.enabled := True;
end;

procedure TfrmMultOrder.lstMultiOrderMeasureItem(Control: TWinControl;
  Index: Integer; var Height: Integer);
var
  TextString,
    TextString2,
    TextString3,
    TextString4: string;
  CellHeight,
    DosageHeight,
    OrderedCount: Integer;
  ARect: TRect;
  procedure ResetRect;
  begin
    with ARect do
    begin
      Left := 0;
      Top := 0;
      Bottom := 0;
      case lstCurrentTab of
        ctUD:
          Right := hdrMultiOrder.Sections[4].Width - 6;
        ctPB, ctIV:
          Right := hdrMultiOrder.Sections[3].Width - 6;
      end;
    end;
  end;

begin
  if VisibleMedList.Count = 0 then Exit;
  if Index = 0 then ItemsHeight := 0;
    //lstmultiorder.
  with TBCMA_MedOrder(VisibleMedList[StrToInt(lstMultiOrder.items[index])]) do
  //with TBCMA_MedOrder(lstMultiOrder.Items.Objects[index]) do
    begin
      TextString := ActiveMedication;
      TextString2 := 'Dummy Text';
      case OrderTypeID of
        otUnitDose:
          OrderedCount := OrderedDrugCount;
        otIV:
          OrderedCount := AdditiveCount + SolutionCount;
      else
        OrderedCount := 1;
      end;

      TextString3 := SpecialInstructions;
      TextString4 := Trim(Dosage) + ', ' + Schedule;
    end;

      ARect := lstMultiOrder.ItemRect(Index);
      ResetRect;

      CellHeight := DrawText(Canvas.Handle, PChar(TextString), Length(TextString),
        ARect, DT_END_ELLIPSIS or DT_NOPREFIX or DT_CALCRECT);

      ResetRect;
      ARect.Left := 7;
      CellHeight := CellHeight + (DrawText(Canvas.Handle, PChar(TextString2), Length(TextString2),
        ARect, DT_END_ELLIPSIS or DT_NOPREFIX or DT_CALCRECT) * OrderedCount);


      ResetRect;
      with TBCMA_MedOrder(VisibleMedList[StrToInt(lstMultiOrder.items[index])]) do
      if OrderTypeID = otUnitDose then
        ARect.Left := 7
      else
        ARect.Left := 14;

      CellHeight := CellHeight + DrawText(Canvas.Handle, PChar(TextString3), Length(TextString3),
        ARect, DT_WORDBREAK or DT_CALCRECT or DT_NOPREFIX or DT_EDITCONTROL or DT_WORD_ELLIPSIS);

      ResetRect;
      case lstCurrentTab of
        ctUD:
          ARect.Right := hdrMultiOrder.Sections[5].Width - 6;
        ctPB, ctIV:
          ARect.Right := hdrMultiOrder.Sections[4].Width - 6;
      end;
      DosageHeight := DrawText(Canvas.Handle, PChar(TextString4), Length(TextString4),
        ARect, DT_WORDBREAK or DT_CALCRECT or DT_NOPREFIX or DT_EDITCONTROL or DT_WORD_ELLIPSIS);

  Height := Max(CellHeight, DosageHeight) + 2;
  ItemsHeight := ItemsHeight + Height;
end;

procedure TfrmMultOrder.lstMultiOrderDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  x,
    ii,
    CellRight,
    TempHeight,
    CurrentTop,
    TitleCount: integer;
  TextString: string;
  CurrentFontColor: TColor;
  ARect: TRect;
  zSpecialInstructions: string;
  outText: string;

begin
  if VisibleMedList.Count > 0 then
    with lstMultiOrder do
      begin

        ARect := Rect;
        CurrentFontColor := Canvas.Font.Color;

        if (TBCMA_MedOrder(VisibleMedList[StrToInt(lstMultiOrder.items[index])]).OrderStatus = 'H') and not (odSelected in State) then
          Canvas.Brush.Color := $00DDDDDD;

        with Canvas do
          begin
            FillRect(ARect);
            Pen.Color := clSilver;
            MoveTo(ARect.Left, ARect.Bottom - 1);
            LineTo(ARect.Right, ARect.Bottom - 1);
          end;

        CellRight := -3;

        case lstCurrentTab of
          ctUD:
            TitleCount := Length(VDLColumnTitles);
          ctPB:
            TitleCount := Length(lstPBColumnTitles);
          ctIV:
            TitleCount := Length(lstIVColumnTitles);
        else
          TitleCount := Length(VDLColumnTitles);
        end;

        for x := 0 to TitleCount - 1 do
          begin
            CellRight := CellRight + hdrMultiOrder.Sections[x].Width;
            Canvas.MoveTo(CellRight, ARect.Bottom - 1);
            Canvas.LineTo(CellRight, ARect.Top);
          end;

        for x := 0 to TitleCount - 1 do
          begin
            if x > 0 then
              ARect.Left := ARect.Right + 2
            else
              ARect.Left := 2;

            ARect.Right := ARect.Left + hdrMultiOrder.Sections[x].Width - 6;

            TextString := '';
            OutText := '';
            with TBCMA_MedOrder(VisibleMedList[StrToInt(lstMultiOrder.items[index])]) do
              case x of
                0:
                  begin
                    case lstCurrentTab of
                      ctUD, ctPB:
                        OutText := ScanStatus;
                      ctIV:
                        OutText := GetOrderStatus(OrderStatus);
                    end;
                  end;
                1:
                  begin
                    OutText := VerifyNurse;
                  end;
                2:
                  begin
                    case lstCurrentTab of
                      ctUD:
                        OutText := SelfMed;
                      ctPB:
                        OutText := ScheduleType;
                      ctIV:
                        OutText := GetIVType(AdministrationUnit);
                    end;
                  end;
                3:
                  begin
                    case lstCurrentTab of
                      ctUD:
                        OutText := ScheduleType;
                      ctPB, ctIV:
                        begin
                          CurrentTop := ARect.Top;
                          TextString := ActiveMedication;
                          TempHeight := DrawText(Canvas.Handle, PChar(TextString), Length(TextString), ARect, DT_END_ELLIPSIS or DT_NOPREFIX);
                          ARect.Left := ARect.Left + 7;
                          for ii := 0 to AdditiveCount - 1 do
                            begin
                              ARect.Top := ARect.Top + TempHeight;
                              TextString := piece(Additives[ii], '^', 3) + ' ' + piece(Additives[ii], '^', 4) + ' ' + piece(Additives[ii], '^', 5);
                              DrawText(Canvas.Handle, PChar(TextString), Length(TextString), ARect, DT_END_ELLIPSIS or DT_NOPREFIX);
                            end;
                          ARect.Left := ARect.Left + 7;
                          for ii := 0 to SolutionCount - 1 do
                            begin
                              ARect.Top := ARect.Top + TempHeight;
                              TextString := piece(Solutions[ii], '^', 3) + ' ' + piece(Solutions[ii], '^', 4) + ' ' + piece(Solutions[ii], '^', 5);
                              DrawText(Canvas.Handle, PChar(TextString), Length(TextString), ARect, DT_END_ELLIPSIS or DT_NOPREFIX);
                            end;

                          ARect.Top := ARect.Top + TempHeight;
                          Canvas.Font.Color := clRed;
                          DrawText(Canvas.Handle, PChar(SpecialInstructions), Length(SpecialInstructions), ARect, DT_WORDBREAK or DT_NOPREFIX or DT_WORD_ELLIPSIS);
                          ARect.Top := CurrentTop;
                        end;
                    end;
                  end;
                4:
                  begin
                    case lstCurrentTab of
                      ctUD:
                        begin
                          CurrentTop := ARect.Top;
                          TextString := ActiveMedication;
                          TempHeight := DrawText(Canvas.Handle, PChar(TextString), Length(TextString), ARect, DT_END_ELLIPSIS or DT_NOPREFIX);
                          ARect.Left := ARect.Left + 7;
                          for ii := 0 to OrderedDrugCount - 1 do
                            begin
                              ARect.Top := ARect.Top + TempHeight;
                              TextString := OrderedDrugs[ii].Name;
                              DrawText(Canvas.Handle, PChar(TextString), Length(TextString), ARect, DT_END_ELLIPSIS or DT_NOPREFIX);
                            end;
                          ARect.Top := ARect.Top + TempHeight;
                          Canvas.Font.Color := clRed;
                          if Copy(SpecialInstructions, 1, 1) = '!' then
                            zSpecialInstructions := Copy(SpecialInstructions, 2, length(SpecialInstructions) - 1)
                          else
                            zSpecialInstructions := SpecialInstructions;
                          DrawText(Canvas.Handle, PChar(zSpecialInstructions), Length(zSpecialInstructions), ARect, DT_WORDBREAK or DT_NOPREFIX or DT_WORD_ELLIPSIS);
                          ARect.Top := CurrentTop;
                          zSpecialInstructions := '';
                        end;
                      ctPB, ctIV:
                        begin
                          TextString := Trim(Dosage) + ', ' + Schedule;
                          DrawText(Canvas.Handle, PChar(TextString), Length(TextString), ARect, DT_WORDBREAK or DT_NOPREFIX or DT_WORD_ELLIPSIS);
                        end;
                    end;
                  end;

                5:
                  begin
                    case lstCurrentTab of
                      ctUD:
                        begin
                          TextString := Trim(Dosage) + ', ' + Schedule;
                          DrawText(Canvas.Handle, PChar(TextString), Length(TextString), ARect, DT_WORDBREAK or DT_NOPREFIX or DT_WORD_ELLIPSIS);
                        end;
                      ctPB, ctIV:
                        OutText := Route;
                    end;
                  end;
                6:
                  case lstCurrentTab of
                    ctUD:
                      OutText := Route;
                    ctPB:
                      OutText := DisplayVADate(AdministrationTime);
                    ctIV:
                      begin
                        TempHeight := DrawText(Canvas.Handle, PChar(DisplayVADateYearTime(TimeLastAction)), Length(DisplayVADateYearTime(TimeLastAction)), ARect, DT_END_ELLIPSIS or DT_NOPREFIX);
                        ARect.Top := ARect.Top + TempHeight;
                        OutText := GetLastActivityStatus(LastActivityStatus);
                      end;
                  end;
                7:
                  case lstCurrentTab of
                    ctUD:
                      if ScheduleType = 'C' then OutText := DisplayVADate(AdministrationTime);
                    ctPB:
                      begin
                        TempHeight := DrawText(Canvas.Handle, PChar(DisplayVADateYearTime(TimeLastAction)), Length(DisplayVADateYearTime(TimeLastAction)), ARect, DT_END_ELLIPSIS or DT_NOPREFIX);
                        ARect.Top := ARect.Top + TempHeight;
                        OutText := GetLastActivityStatus(LastActivityStatus);
                      end;
                  end;
                8:
                  case lstCurrentTab of
                    ctUD:
                      begin
                        TempHeight := DrawText(Canvas.Handle, PChar(DisplayVADateYearTime(TimeLastAction)), Length(DisplayVADateYearTime(TimeLastAction)), ARect, DT_END_ELLIPSIS or DT_NOPREFIX);
                        ARect.Top := ARect.Top + TempHeight;
                        OutText := GetLastActivityStatus(LastActivityStatus);
                        //Force Horizontal Scrollbar on
                        //Perform(LB_SETHORIZONTALEXTENT, ARect.Right, 0);
                      end;
                  end;
              end;

            case lstCurrentTab of
              ctUD:
                case x of
                  0, 1, 2, 3, 6, 7, 8:
                    DrawText(Canvas.Handle, PChar(OutText), Length(OutText), ARect, DT_END_ELLIPSIS or DT_NOPREFIX);
                end;
              ctPB, ctIV:
                case x of
                  0, 1, 2, 5, 6, 7:
                    DrawText(Canvas.Handle, PChar(OutText), Length(OutText), ARect, DT_END_ELLIPSIS or DT_NOPREFIX);
                end;
            end;
            Canvas.Font.Color := CurrentFontColor;
            ARect.Right := ARect.Right + 4;
          end;
      end;
end;


procedure TfrmMultOrder.hdrMultiOrderSectionResize(
  HeaderControl: THeaderControl; Section: THeaderSection);
var
  TotalWidth,
    TempWidth,
    ii: Integer;
begin
  TotalWidth := 0;

  with HeaderControl.Sections do
    begin
      for ii := 0 to Count - 1 do
        begin
          TempWidth := ((Count - (ii + 1)) * 5);
          items[ii].maxwidth := HeaderControl.width - (TempWidth + TotalWidth);
          TotalWidth := TotalWidth + Items[ii].Width
        end;
    end;
  lstMultiOrder.Repaint;
end;

procedure TfrmMultOrder.lstMultiOrderClick(Sender: TObject);
begin
  if lstMultiOrder.ItemIndex <> -1 then
    btnOk.Enabled := True;

end;

End.

