unit PRNEffectiveness;
{
================================================================================
*	File:  PRNEffectiveness.PAS
*
*	Application:  Bar Code Medication Administration
*	Revision:     $Revision: 21 $  $Modtime: 5/02/02 2:37p $
*	Description:  This is the form for logging a PRN Effectiveness for a PRN order.
*
*	Notes:
*
*
================================================================================
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, Buttons, ExtCtrls,
  BCMA_Objects, BCMA_Util, Grids, mVitals;

type
  TPRNListColumnTypes = (
    //													ltMedLogIEN, ltPatientIEN,
    ltLocation,
    ltDivision,
    ltAdministrationTime,
    ltAdministeredBy,
    ltReasonGiven
    );

const
  PRNListColumnTitles: array[TPRNListColumnTypes] of string =
  (
    //											'MedLogIEN', 'PatientIEN',
    'Location', 'Division',
    'Administration Time', 'Administered By', 'Reason Given'
    );

  PRNListColumnWidths: array[TPRNListColumnTypes] of integer =
  (
    //											35, 35,
    100, 100,
    150, 120, 150
    );

type
  TfrmPRNEffectiveness = class(TForm)
    pnlOrderInfo: TPanel;
    lblActiveMedication: TLabel;
    lblDispensedDrug: TLabel;
    Label3: TLabel;
    mmoSpecialInstructions: TMemo;
    pnlButton: TPanel;
    Label2: TLabel;
    memAddComment: TMemo;
    lblMaxChar: TLabel;
    btnExit: TButton;
    pnlPRNList: TPanel;
    lblSelectedAdministration: TLabel;
    lstDispensedDrugs: TListBox;
    btnOK: TButton;
    btnCancel: TButton;
    btnMedHistory: TButton;
    lvwPRNList: TListView;
    lblPRNList: TLabel;
    lblUnitsGiven: TLabel;
    fraVitals1: TfraVitals;
    GroupBox1: TGroupBox;
    cbxPainScore: TComboBox;
    lblPainScore: TLabel;
    lblDateTime: TLabel;
    edtDateTime: TEdit;
    procedure FormCreate(Sender: TObject);
    (*
      Allocates memory and initializes variables and parameters.
    *)

    procedure FormDestroy(Sender: TObject);
    (*
      Frees allocated memory.
    *)

    procedure FormShow(Sender: TObject);
    (*
      Loads readonly fields with properties from the current MedOrder.  Loads
      a list of the PRN orders that need effectiveness comments.  Sets focus
      to the PRN List.
    *)

    procedure pnlButtonResize(Sender: TObject);
    (*
      Resets the positions of the OK and Cancel buttons whenever the the form
      is resized, to ensure that they will always be visible.
    *)

    procedure btnOKClick(Sender: TObject);
    procedure memAddCommentKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btnMedHistoryClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure lvwPRNListColumnClick(Sender: TObject; Column: TListColumn);
    procedure lvwPRNListDblClick(Sender: TObject);
    procedure lvwPRNListKeyPress(Sender: TObject; var Key: Char);
    procedure cbxPainScoreChange(Sender: TObject);
    procedure cbxPainScoreKeyPress(Sender: TObject; var Key: Char);
    procedure edtDateTimeExit(Sender: TObject);
    procedure lvwPRNListSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure edtDateTimeChange(Sender: TObject);
    (*
      Checks that the Comment memo is not blank, then uses RPC
      'PSB TRANSACTION' to log the user input.
    *)

  private
    { Private declarations }
    procedure RefreshPRNList;
    //    function PRNListCompare(Item1, Item2: pointer): Integer;
  public
    { Public declarations }
    MedOrder: TBCMA_MedOrder; {Pointer to the current MedOrder}
    OrderPRNList: TStringList; {Pointer to the current PRNList}

    procedure Execute(PatientMedOrder: TBCMA_MedOrder);
    (*
      Creates the form, retrieves the list of PRN's needing Effectiveness
      comments and shows the form modally.  Finally, the form is closed and
      memory is freed.
    *)

  end;

type
  TPRNSortType = (stAscending, stDescending);

var
  frmPRNEffectiveness: TfrmPRNEffectiveness;
  PRNSortCol: lstPRNListColumnTypes;
  PRNSortType: TPRNSortType;
  PainScoreMNowDateTime: string;

implementation
{$R *.DFM}

uses
  Math, BCMA_Startup, BCMA_Common, MFunStr, BCMA_Main, ReportRequest;

function PRNListCompare(Item1, Item2: pointer): Integer;
begin
  result := 1;

  case PRNSortCol of
    ctPRNActiveMedication:
      result := AnsiCompareStr(TBCMA_PRNEffectList(Item1).AdministeredDrug,
        TBCMA_PRNEffectList(Item2).AdministeredDrug);
    ctPRNUnitsGiven:
      result := AnsiCompareStr(TBCMA_PRNEffectList(Item1).UnitsGiven,
        TBCMA_PRNEffectList(Item2).UnitsGiven);
    ctPRNAdminTime:
      result := AnsiCompareStr(TBCMA_PRNEffectList(Item1).AdminDateTime,
        TBCMA_PRNEffectList(Item2).AdminDateTime);
    ctPRNReasonGiven:
      result := AnsiCompareStr(TBCMA_PRNEffectList(Item1).PRNReason,
        TBCMA_PRNEffectList(Item2).PRNReason);
    ctPRNAdministeredBy:
      result := AnsiCompareStr(TBCMA_PRNEffectList(Item1).AdministeredBy,
        TBCMA_PRNEffectList(Item2).AdministeredBy);
    ctPRNLocation:
      result := AnsiCompareStr(TBCMA_PRNEffectList(Item1).PatientLocation,
        TBCMA_PRNEffectList(Item2).PatientLocation);

  end;

  if PRNSortType = stDescending then
  begin
    if result = 1 then
      result := -1
    else if result = -1 then
      result := 1;
  end;

end;

procedure TfrmPRNEffectiveness.Execute(PatientMedOrder: TBCMA_MedOrder);
begin
  with TfrmPRNEffectiveness.create(application) do
  try
    MedOrder := PatientMedOrder;

    with BCMA_Patient do
    begin
      if MedOrder <> nil then LoadPRNEffectiveness(MedOrder.OrderNumber);

      if PRNEffectList.Count > 0 then ShowModal
      else DefMessageDlg('There are no PRN''s needing Effectivenesses!', mtError, [mbOK], 0);
    end;
  finally
    free;
  end;
end;

procedure TfrmPRNEffectiveness.FormCreate(Sender: TObject);
var
  ii: TPRNListColumnTypes;
  x: integer;
begin
  if isRestricted then
  begin
    lblPainScore.Enabled := False;
    cbxPainScore.Enabled := False;
    lblDateTime.Enabled := False;
    edtDateTime.Enabled := False;
    label2.Enabled := False;
    lblMaxChar.Enabled := False;
    memAddComment.Enabled := False;
  end;

  OrderPRNList := TStringList.create;
  memAddComment.clear;

  with cbxPainScore do
  begin
    AddItem('0 - No Pain', ptr(0));
    AddItem('1 - Slightly Uncomfortable', ptr(1));
    for x := 2 to 9 do
      AddItem(IntToStr(x), ptr(x));
    AddItem('10 - Worst Imaginable', ptr(10));
    AddItem('99 - Unable to Respond', ptr(99));
  end;

  fraVitals1.LoadVitals;
end;

procedure TfrmPRNEffectiveness.FormShow(Sender: TObject);
begin
  PRNSortCol := ctPRNAdminTime;
  PRNSortType := stDescending;
  RefreshPRNList;
end;

procedure TfrmPRNEffectiveness.btnOKClick(Sender: TObject);
var
  MedLogIEN,
    cmdString,
    PainInfo: string;

  MultList: TStringList;
begin
  if lvwPRNList.SelCount = 0 then exit;
  if (TrimLeft(memAddComment.Text) = '') and (TBCMA_PRNEffectList(BCMA_Patient.PRNEffectList.Items[lvwPRNList.ItemIndex]).RequirePainScore <> 1) then
  begin
    DefMessageDlg('You Must Enter a Comment!', mtError, [mbOK], 0);
    memAddComment.setFocus;
  end
  else
  begin
    cmdString := IntToStr(Integer(lvwPRNList.Items[lvwPRNList.Selected.index].data)) + '^PRN EFFECTIVENESS';
    if TBCMA_PRNEffectList(BCMA_Patient.PRNEffectList.Items[lvwPRNList.ItemIndex]).RequirePainScore = 1 then
    begin
      PainInfo := 'Pain Score of ' +
      IntToStr(Integer(cbxPainScore.Items.Objects[cbxPainScore.ItemIndex])) +
      ' entered into Vitals via BCMA at ' +
        FormatFMDateTime('MM/DD/YYYY@HH:NN', StrToFloat(PainScoreMNowDateTime));
      if StripString(memAddComment.Text, False) <> '' then
        PainInfo := ' - ' + PainInfo;
    end;

    MultList := TStringList.create;
    with MultList do
    try
      add(StripString(memAddComment.Text + PainInfo));
      with BCMA_Broker do
        if CallServer('PSB TRANSACTION', [cmdString + '^' + BCMA_User.InstructorDUZ], MultList) then
          if piece(results[0], '^', 1) = '-1' then
            DefMessageDlg(piece(results[0], '^', 2), mtError, [mbOK], 0)
          else
            with BCMA_Patient do
            begin
              TBCMA_PRNEffectList(PRNEffectList.Items[lvwPRNList.Selected.index]).Free;
              PRNEffectList.Delete(lvwPRNList.Selected.Index);
            end;

    finally
      free;
    end;
    if cbxPainScore.ItemIndex <> -1 then
      SendVitals(IntToStr(Integer(cbxPainScore.Items.Objects[cbxPainScore.ItemIndex])),
        PainScoreMNowDateTime);

    RefreshPRNList;
    cbxPainScore.ItemIndex := -1;
    cbxPainScore.Enabled := False;
    edtDateTime.Enabled := False;
    edtDateTime.Clear;
  end;
end;

procedure TfrmPRNEffectiveness.FormDestroy(Sender: TObject);
begin
  OrderPRNList.free;
end;

procedure TfrmPRNEffectiveness.pnlButtonResize(Sender: TObject);
begin
  //  btnOK.Width := btnCancel.Width;
  //  btnCancel.Left := pnlButton.ClientWidth -
  //    (btnCancel.Width + self.BorderWidth * 2);
  //  btnOK.Left := btnCancel.Left - (btnOK.Width + self.BorderWidth);
end;

procedure TfrmPRNEffectiveness.RefreshPRNList;
var
  x: integer;
begin
  btnOK.Enabled := False;
  btnMedHistory.Enabled := False;
  btnCancel.Enabled := False;
  lvwPRNList.tag := 0;
  lvwPRNList.Clear;
  mmoSpecialInstructions.Clear;
  lblActiveMedication.caption := 'Active Medication:';
  lstDispensedDrugs.Clear;
  memAddComment.Clear;
  memAddComment.enabled := False;

  with BCMA_Patient do
    for x := 0 to PRNEffectList.Count - 1 do
    begin
      with TBCMA_PRNEffectList(PRNEffectList.Items[x]) do
        lvwPRNList.AddItem(AdministeredDrug, Ptr(StrToInt(MedLogIEN)));
      lvwPRNList.Items[x].SubItems.Add(TBCMA_PRNEffectList(PRNEffectList.Items[x]).UnitsGiven);
      lvwPRNList.Items[x].SubItems.Add(DisplayVADateYearTime(TBCMA_PRNEffectList(PRNEffectList.Items[x]).AdminDateTime));
      lvwPRNList.Items[x].SubItems.Add(TBCMA_PRNEffectList(PRNEffectList.Items[x]).PRNReason);
      lvwPRNList.Items[x].SubItems.Add(TBCMA_PRNEffectList(PRNEffectList.Items[x]).AdministeredBy);
      lvwPRNList.Items[x].SubItems.Add(TBCMA_PRNEffectList(PRNEffectList.Items[x]).PatientLocation);
    end;
  lvwPRNList.tag := 1;
end;

procedure TfrmPRNEffectiveness.memAddCommentKeyUp(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if ((memAddComment.Text = '') and (cbxPainScore.Enabled = False) or ((cbxPainScore.enabled = true) and
    ((cbxPainScore.ItemIndex = -1) or (edtDateTime.text = '')))) then
    btnOK.Enabled := False
  else
    btnOK.Enabled := True;
end;

procedure TfrmPRNEffectiveness.btnMedHistoryClick(Sender: TObject);
begin
  if lvwPRNList.SelCount <> 0 then
    with BCMA_Patient do
      with TBCMA_PRNEffectList(PRNEffectList.Items[lvwPRNList.Selected.index]) do
        MedicationHistoryReport(BCMA_Patient.IEN, OrderableItemIEN, OrderNumber);

end;

procedure TfrmPRNEffectiveness.btnCancelClick(Sender: TObject);
begin
  RefreshPRNList;
  cbxPainScore.ItemIndex := -1;
  cbxPainScore.Enabled := False;

  edtDateTime.Enabled := False;
  edtDateTime.Clear;

  lvwPRNList.setfocus;
end;

procedure TfrmPRNEffectiveness.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
  msg: string;
begin
  msg := 'The documentation for the selected administration is incomplete and will ' +
    'not be saved!' + #13#13 + 'Click OK to close without saving or click the Cancel button to return to the PRN Effectiveness Log.';

  if ((memAddComment.Text <> '') or (cbxPainScore.ItemIndex <> -1)) then
    if DefMessageDlg(msg, mtWarning, [mbOK, mbCancel], 0) = idCancel then
      CanClose := False;
end;

procedure TfrmPRNEffectiveness.lvwPRNListColumnClick(Sender: TObject;
  Column: TListColumn);
begin
  if lstPRNListColumnTypes(Column.Index) = PRNSortCol then
    if PRNSortType = stAscending then
      PRNSortType := stDescending
    else if PRNSortType = stDescending then
      PRNSortType := stAscending;

  PRNSortCol := lstPRNListColumnTypes(Column.Index);

  BCMA_Patient.PRNEffectList.Sort(PRNListCompare);
  RefreshPRNList;
end;

procedure TfrmPRNEffectiveness.lvwPRNListDblClick(Sender: TObject);
begin
  if lvwPRNList.SelCount <> 0 then
    with BCMA_Patient do
      with TBCMA_PRNEffectList(PRNEffectList.Items[lvwPRNList.Selected.index]) do
        DisplayOrder(IEN, OrderNumber);

end;

procedure TfrmPRNEffectiveness.lvwPRNListKeyPress(Sender: TObject;
  var Key: Char);
begin
  if key = chr(VK_RETURN) then
    lvwPRNListDblClick(lvwPRNList);

end;

procedure TfrmPRNEffectiveness.cbxPainScoreChange(Sender: TObject);
begin
  if isRestricted or (edtDateTime.text = '') or ((cbxPainScore.Enabled = True) and (cbxPainScore.ItemIndex = -1)) then
    btnOK.Enabled := False
  else
    btnOK.Enabled := True;

end;

procedure TfrmPRNEffectiveness.cbxPainScoreKeyPress(Sender: TObject;
  var Key: Char);
begin
  //Prevent a backspace.  Apparently under W2K, a backspace will set
  //itemindex to -1.
  if Key = #8 then
    Key := #0;
end;

procedure TfrmPRNEffectiveness.edtDateTimeExit(Sender: TObject);
var
  ss,
    MActionDateTime,
    NowMDateTime,
    zNow: string;

begin
  if edtDateTime.text <> '' then
  begin
    ss := edtDateTime.text;

    MActionDateTime := ValidMDateTime(ss);
    PainScoreMNowDateTime := MActionDateTime;
    zNow := 'N';
    if edtDateTime.text <> zNow then
      NowMDateTime := ValidMDateTime(zNow);

    if NowMDateTime = '' then NowMDateTime := MActionDateTIme;

    if MActionDateTime <> '' then
    begin
      if StrToFloat(MActionDateTime) > StrToFloat(NowMDateTime) then
      begin
        DefMessageDlg('The action date and time can''t be in the future.', mtError,
          [mbOk], 0);
        edtDateTime.setFocus;
        exit;
      end;
      if StrToFloat(MActionDateTime) < StrToFloat(TBCMA_PRNEffectList(BCMA_Patient.PRNEffectList.Items[lvwPRNList.ItemIndex]).AdminDateTime) then
      begin
        DefMessageDlg('The action date and time can''t be prior to the administration date and time.', mtError,
          [mbOk], 0);
        edtDateTime.setFocus;
        exit;
      end;

      edtDateTime.text := ss;
    end
    else
      edtDateTime.setFocus;
  end;
end;

procedure TfrmPRNEffectiveness.lvwPRNListSelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
var
  x: integer;
  zDatetime: string;
begin
  if lvwPRNList.tag = 0 then exit;

  if lvwPRNList.SelCount = 0 then
  begin
    btnOK.Enabled := False;
    btnMedHistory.Enabled := False;
    btnCancel.Enabled := False;
    mmoSpecialInstructions.Clear;
    lblActiveMedication.caption := 'Active Medication:';
    lstDispensedDrugs.Clear;
    memAddComment.Clear;
    memAddComment.enabled := False;
    Exit;
  end;
  if not IsRestricted then
  begin
    btnCancel.Enabled := True;
    memAddComment.enabled := True;
  end;
  btnMedHistory.Enabled := True;
  memAddComment.Clear;

  with TBCMA_PRNEffectList(BCMA_Patient.PRNEffectList.Items[Item.index]) do
  begin
    mmoSpecialInstructions.Text := SpecialInsructions;
    lblActiveMedication.caption := 'Active Medication: ' + AdministeredDrug;
    if Not IsRestricted then
    begin
      cbxPainScore.Enabled := (RequirePainScore = 1);
      edtDateTime.Enabled := (RequirePainScore = 1);
    end;
    edtDateTime.Clear;
    cbxPainScore.ItemIndex := -1;
    lstDispensedDrugs.Clear;
    PainScoreMNowDateTime := '';
    for x := 0 to DispensedDrugs.Count - 1 do
      lstDispensedDrugs.AddItem(piece(DispensedDrugs[x], '^', 2), nil);
    for x := 0 to Additives.count - 1 do
      lstDispensedDrugs.AddItem(piece(Additives[x], '^', 2), nil);
    for x := 0 to Solutions.count - 1 do
      lstDispensedDrugs.AddItem(piece(Solutions[x], '^', 2), nil);
    if RequirePainScore = 1 then
    begin
      Repaint;
      zDateTime := 'N';
      PainScoreMNowDateTime := ValidMDateTime(zDateTime);
      edtDateTime.Text := zDateTime;
      lblMaxChar.Caption := '(80 Characters Maximum)';
      memAddComment.MaxLength := 80;
    end
    else
    begin
      lblMaxChar.Caption := '(150 Characters Maximum)';
      memAddComment.MaxLength := 150;
    end;
  end;

end;

procedure TfrmPRNEffectiveness.edtDateTimeChange(Sender: TObject);
begin
  if isRestricted or (edtDateTime.Text = '') or ((cbxPainScore.Enabled = True) and (cbxPainScore.ItemIndex = -1)) then
    btnOK.Enabled := False
  else
    btnOK.Enabled := True;
end;

end.

