Unit WardStock;
{
================================================================================                        
*	File:  WardStock.PAS
*
*	Application:  Bar Code Medication Administration
*	Revision:     $Revision: 5 $  $Modtime: 3/04/02 2:14p $
*
*	Description:  This is a form for scanning additives and solutions for a
*               WardStock order.

*	Notes:
*
================================================================================
}

Interface

Uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, BCMA_Objects, CheckLst, BCMA_Util;

Type
  TfrmWardStock = Class(TForm)
 (*
   This is a form for scanning multiple additives and soltuions for a wardstock
 *)
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    Panel5: TPanel;
    pnlScannerStatus: TPanel;
    pnlScannerIndicator: TPanel;
    pnlScannerInput: TPanel;
    lblStannerStatus: TLabel;
    lblScannerStatusReady: TLabel;
    lblScanMedication: TLabel;
    edtScannerInput: TEdit;
    btnCancel: TButton;
    btnDone: TButton;
    lstWardStock: TListBox;
    Label2: TLabel;
    Label1: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Procedure FormCreate(Sender: TObject);
  (*
    Allocates memory and initializes variables.
  *)

    Procedure FormDestroy(Sender: TObject);
  (*
    Frees memory allocated.
  *)

    Procedure FormShow(Sender: TObject);
  (*
    Set focus to edtScannerInput
  *)

    Procedure edtScannerInputEnter(Sender: TObject);
  (*
    Sets pnlScannerIndicator.Color to clLime and sets
    lblScannerStatusReady.Caption to 'Ready', indicating that the scanner
    is ready to receive input.
  *)

    Procedure edtScannerInputExit(Sender: TObject);
  (*
    Sets pnlScannerIndicator.Color to clRed and sets
    lblScannerStatusReady.Caption to 'Not Ready', indicating that the
    scanner is not ready to receive input.
  *)

    Procedure edtScannerInputKeyPress(Sender: TObject; Var Key: Char);
  (*
    Waits for an <Enter> key, then calls method MarkDrugAsGiven to validate
    the edtScannerInput.text value.
  *)

    Procedure pnlScannerIndicatorClick(Sender: TObject);
  (*
    Sets focus to field edtScannerInput.
  *)

    Procedure FormCloseQuery(Sender: TObject; Var CanClose: Boolean);
    {
      Checks to see if the form can be closed
    }

    Procedure btnDoneClick(Sender: TObject);
    {
      Set variable CloseFrm = True, if edtScannerInput.text is not null, then
      call method MarkDrugAsGiven
    }

    procedure btnCancelClick(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure edtScannerInputKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure edtScannerInputKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    {
      Sets CloseFrm = true
    }

  Private
  { Private declarations }
    ScanCount: integer;

    function MarkDrugAsGiven(scanIEN: String): Boolean;
    {
      Calls method ScanDrug, the clears and sets focus to edtScannerInput
    }

  Public
  { Public declarations }
    MedOrder: TBCMA_MedOrder;
    Drugidx: integer;
    UAS_LogState: TUASLogAction;  {JK 5/16/2008}
    Function ScanDrug(ScanIEN: String; AlreadyValid: Boolean = False): Boolean;
    {
      Calls method isValidDrug to check to see if the drug is valid, then adds it
      to the appropriate TStringList, and populates the list box.
    }

  End;

Var
  frmWardStock: TfrmWardStock;
  Mult_ScannedDrug: TBCMA_DispensedDrug;
  CloseFrm: Boolean;
  zAdditives,
  zSolutions: TStringList;

Implementation

{$R *.DFM}
Uses
  BCMA_Startup,
  BCMA_Common,
  MFunStr,
  BCMA_Main;

Procedure TfrmWardStock.FormCreate(Sender: TObject);
Begin
  Mult_ScannedDrug := TBCMA_DispensedDrug.create(BCMA_Broker);
//  clbxDoses.clear;
  ScanCount := 0;
  CloseFrm := True;
  zAdditives := TStringList.Create;
  zSolutions := TStringList.Create;
  frmMain.WardStockInProc := True;
  UAS_LogState := LA_OkToLog; {JK 5/16/2008}
End;

Procedure TfrmWardStock.FormShow(Sender: TObject);
Begin
  edtScannerInput.setFocus;
End;

procedure TfrmWardStock.FormActivate(Sender: TObject);
begin
  edtScannerInputEnter(Sender);
end;


Procedure TfrmWardStock.FormDestroy(Sender: TObject);
Begin
  Mult_ScannedDrug.free;
  frmMain.WardStockInProc := False;
End;

Procedure TfrmWardStock.FormCloseQuery(Sender: TObject;
  Var CanClose: Boolean);
Begin
  CanClose := CloseFrm;
  CloseFrm := True;
End;

Procedure TfrmWardStock.edtScannerInputEnter(Sender: TObject);
Begin
  pnlScannerIndicator.Color := clLime;
  lblScannerStatusReady.Caption := 'Ready';
  StopKeyboardTimer;
End;

Procedure TfrmWardStock.edtScannerInputExit(Sender: TObject);
Begin
  pnlScannerIndicator.Color := clRed;
  lblScannerStatusReady.Caption := 'Not Ready';
End;

function TfrmWardStock.MarkDrugAsGiven(scanIEN: String): Boolean;
Begin
  if not ScanDrug(scanIEN) then
    Result := False
  else
    Result := True;

  edtScannerInput.clear;
  edtScannerInput.setFocus;

End;

Procedure TfrmWardStock.edtScannerInputKeyPress(Sender: TObject;
  Var Key: Char);
Begin
  {disable Cut and Past}
  if Key = #22 then key := #0;

  if (edtScannerInput.text <> '') and (key = chr(VK_RETURN)) then begin
    //Logic Added to Deal with Keyed Bar Codes
    StopKeyboardTimer;
    if KeyedBarCode then //begin
      //LogUnableToScan('', '', '', '', 'M', 1); {Removed 7/8/2008 JK per CQ #116}
      KeyedBarCode := False;
    //end else
      //LogUnableToScan('', '', '', '', 'M', 2);  {Removed 7/17/2008 JK per CQ #116}

    MarkDrugAsGiven(edtScannerInput.Text);
  end;
End;

Procedure TfrmWardStock.pnlScannerIndicatorClick(Sender: TObject);
Begin
  edtScannerInput.SetFocus;
End;

procedure TfrmWardStock.btnDoneClick(Sender: TObject);
var
  CRKey: Char;
begin
  CRKey := chr(VK_RETURN);
  if Trim(edtScannerInput.Text) <> '' then begin
    edtScannerInputKeyPress(Self, CRKey);    {JK 8/24/2008}
    CloseFrm := False;
  end else begin
  
    CloseFrm := True;

    if edtScannerInput.text <> '' then
      if MarkDrugAsGiven(edtScannerInput.text) then
        CloseFrm := True
      else
        CloseFrm := False;
//  else begin
//    DefMessageDlg('You did not specify a barcode. Re-enter code or press cancel to quit',
//      mtInformation, [mbOK], 0);
//    CloseFrm := False;
//  end;
  end;


End;

procedure TfrmWardStock.btnCancelClick(Sender: TObject);
begin
  DefMessageDlg('Order Administration Cancelled!', mtWarning, [mbOK], 0);
  UAS_LogState := LA_Cancelled; {JK 5/16/2008}  
  CloseFrm := True;
end;


function TfrmWardStock.ScanDrug(ScanIEN: String; AlreadyValid: Boolean): Boolean;
begin
  with BCMA_ScannedDrug do begin
    if AlreadyValid = False then
      if not isValidDrug(ScanIEN) then begin
        Result := False;
        Exit
      end;
    if piece(ResultString, '^', 1) = 'ADD' then
      zAdditives.Add(ResultString)
    else if piece(ResultString, '^', 1) = 'SOL' then
        zSolutions.Add(ResultString);

    lstWardStock.Items.Add(BCMA_ScannedDrug.Name + ', ' + Dose);
    Result := True;

  end;
end;

procedure TfrmWardStock.FormDeactivate(Sender: TObject);
begin
  edtScannerInputExit(Sender);
end;

procedure TfrmWardStock.edtScannerInputKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  {disable cut and paste}
  if key = VK_Insert then
    key := 0;
end;

procedure TfrmWardStock.edtScannerInputKeyUp(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if edtScannerInput.Text <> '' then
  begin
    if (KeyBoardTimer = nil) or (KeyedBarCode = False) then StartKeyboardTimer;
  end
  else
  begin
    StopKeyboardTimer;
    KeyedBarCode := False;
  end;
end;

End.

