unit fEdtMedLogAdminSelect;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, BCMA_Startup, BCMA_Objects, DateUtils;

function EditMedLogSelectAdministration: Boolean;

type
  TfrmEditMedLogAdminSelect = class(TForm)
    lblDate: TLabel;
    lvwAdministrations: TListView;
    btnDown: TButton;
    edtDate: TEdit;
    btnDatePicker: TButton;
    btnUp: TButton;
    btnOK: TButton;
    btnCancel: TButton;
    lblPatientNameCaption: TLabel;
    lblSSNCaption: TLabel;
    lblPatientName: TLabel;
    lblSSN: TLabel;
    procedure edtDateExit(Sender: TObject);
    procedure edtDateEnter(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnDownClick(Sender: TObject);
    procedure btnUpClick(Sender: TObject);
    procedure lvwAdministrationsChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure btnOKClick(Sender: TObject);
    procedure btnDatePickerClick(Sender: TObject);
    procedure edtDateKeyPress(Sender: TObject; var Key: Char);
    procedure edtDateChange(Sender: TObject);
  private
    { Private declarations }
    function CheckDate(DateIn: string): Boolean;
    function RefreshAdministrations: Boolean;
  public
    { Public declarations }
  end;

var
  frmEditMedLogAdminSelect: TfrmEditMedLogAdminSelect;
  SelectedDate: TDateTime;
implementation

{$R *.dfm}
uses
  BCMA_Common,
  BCMA_Util,
  BCMA_Main,
  fEditMedLog,
  fDateTimePicker;

function EditMedLogSelectAdministration: Boolean;
begin
  with TfrmEditMedLogAdminSelect.create(application) do
  try
    edtDate.Text := 'N';
    CheckDate(edtDate.Text);
    RefreshAdministrations;
    ShowModal;
  finally
    free;
  end;
end;

function TfrmEditMedLogAdminSelect.RefreshAdministrations: Boolean;
var
  MultList: TStringList;
  x: integer;
begin
  Result := False;
  MultList := TStringList.Create;
  lvwAdministrations.Clear;
  lvwAdministrations.Repaint;
  btnOK.Enabled := False;
  with MultList do
  begin
    Add('ADMLKUP');
    Add(BCMA_Patient.IEN);
    Add(piece(DateTimeToMDateTime(SelectedDate), '.', 1));
  end;
  frmMain.StatusBar.Panels[0].Text := 'Retrieving Administrations...';
  frmMain.StatusBar.Repaint;

  if (BCMA_Broker <> nil) then
    with BCMA_Broker do
      if CallServer('PSB MED LOG LOOKUP', ['~!#null#~!'], MultList) then
      begin
        if (Results.Count = 0) or (Results.Count - 1 <> StrToIntDef(Results[0], -1)) then
          DefMessageDlg(ErrIncompleteData, mtError, [mbOK], 0)
        else if piece(Results[1], '^', 1) = '-1' then
          lblDate.Caption := 'Searching Date: ' + edtDate.Text + ' - ' + piece(Results[1], '^', 2)
        else
        begin
          Result := True;
          Results.Delete(0);
          for x := 0 to Results.Count - 1 do
            with lvwAdministrations do
            begin
              AddItem(Piece(Results[x], '^', 6), ptr(StrToInt(piece(Results[x], '^', 1))));
              if Piece(Results[x], '^', 4) <> '' then
                Items[x].SubItems.Add(Piece(Results[x], '^', 3) + '  (' + Piece(Results[x], '^', 4) + ')')
              else
                Items[x].SubItems.Add(Piece(Results[x], '^', 3));

              //if Piece(Results[x], '^', 4) <> '' then
              //  AddItem(Piece(Results[x], '^', 3) + '  (' + Piece(Results[x], '^', 4) + ')', ptr(StrToInt(piece(Results[x], '^', 1))))
              //else
              //  AddItem(Piece(Results[x], '^', 3), ptr(StrToInt(piece(Results[x], '^', 1))));
              Items[x].SubItems.Add(GetLastActivityStatus(piece(Results[x], '^', 5)));
              Items[x].SubItems.Add(DisplayVADateYearTime(piece(Results[x], '^', 7)));
              Items[x].SubItems.Add(piece(Results[x], '^', 8));
            end;
        end;
      end;
  MultList.Free;
  frmMain.StatusBar.Panels[0].Text := '';
end;

procedure TfrmEditMedLogAdminSelect.edtDateExit(Sender: TObject);
begin
  if (edtDate.Tag = 1) or (edtDate.Text = '') then exit;
  if CheckDate(edtDate.Text) then RefreshAdministrations
  else
  begin
//    edtDate.Clear;
    edtDate.SetFocus;
  end;
end;

procedure TfrmEditMedLogAdminSelect.edtDateEnter(Sender: TObject);
begin
//  edtDate.tag := 0;
end;

function TfrmEditMedLogAdminSelect.CheckDate(DateIn: string): Boolean;
var
  MDate: Double;
  NowMDate: string;
begin
  //if null, then add space, a null value causes an M error.
  // we are relying the M side to return the error message to display to the user

  if DateIn = '' then DateIn := ' ';
  MDate := -1;
  MDate := StrToFloatDef(piece(ValidMDateTime(DateIn), '.', 1), -1);
//  if MDate = -1 then exit;
  if MDate <> -1 then
  begin
    NowMDate := 'N';
    NowMDate := Piece(ValidMDateTime(NowMDate), '.', 1);
    if MDate > StrToFloat(piece(NowMDate, '^', 1)) then
    begin
      DefMessageDlg('The date can''t be in the future.', mtError, [mbOk], 0);
      DateIn := DateTimeToMDateTime(SelectedDate);
      ValidMDateTime(DateIn);
      edtDate.text := piece(DateIn, '@', 1);
      edtDate.SetFocus;
//      edtDate.Clear;
      edtDate.Tag := 0;
      result := False;
    end
    else
    begin
      SelectedDate := FMDateTimeToDateTime(MDate + 0.0001);
      edtDate.text := piece(DateIn, '@', 1);
      edtDate.tag := 1;
      lblDate.Caption := 'Searching Date: ' + edtDate.Text;
      Result := true;
    end;
  end
  else
  begin
    edtDate.setFocus;
    DateIn := DateTimeToMDateTime(SelectedDate);
    ValidMDateTime(DateIn);
    edtDate.text := piece(DateIn, '@', 1);
    //edtDate.Clear;
    edtDate.tag := 0;
    result := False;
  end;
end;

procedure TfrmEditMedLogAdminSelect.FormCreate(Sender: TObject);
begin
  lblPatientName.Caption := BCMA_Patient.Name;
  lblSSN.Caption := BCMA_Patient.SSN;

  with lvwAdministrations do
  begin
    Column[0].Width := trunc(Width * 0.07);
    Column[1].Width := trunc(Width * 0.5);
    Column[2].Width := trunc(Width * 0.13);
    Column[3].Width := trunc(Width * 0.20);
    Column[4].Width := trunc(Width * 0.09);
  end;
end;

procedure TfrmEditMedLogAdminSelect.btnDownClick(Sender: TObject);
begin
  if CheckDate(piece(DateTimeToMDateTime(SelectedDate - 1), '.', 1)) then
    RefreshAdministrations;
end;

procedure TfrmEditMedLogAdminSelect.btnUpClick(Sender: TObject);
begin
  if CheckDate(piece(DateTimeToMDateTime(SelectedDate + 1), '.', 1)) then
    RefreshAdministrations;
end;

procedure TfrmEditMedLogAdminSelect.lvwAdministrationsChange(
  Sender: TObject; Item: TListItem; Change: TItemChange);
begin
  if lvwAdministrations.ItemIndex <> -1 then btnOk.Enabled := True
  else btnOk.Enabled := False;
end;

procedure TfrmEditMedLogAdminSelect.btnOKClick(Sender: TObject);
begin
  if (edtDate.tag = 0) then
    if not CheckDate(edtDate.Text) then
    begin
      edtDate.Clear;
      edtDate.setFocus;
      exit;
    end;

  if lvwAdministrations.ItemIndex <> -1 then
    if EditMedLogExecute(IntToStr(Integer(lvwAdministrations.Items[lvwAdministrations.Selected.index].data))) then
      RefreshAdministrations;
end;

procedure TfrmEditMedLogAdminSelect.btnDatePickerClick(Sender: TObject);
var
  zDateTime, zMDateTime: string;
begin
  zMDateTime := DateTimePickerExecute(zDateTime, False, DateTimeToMDateTime(SelectedDate), True);
  if zMDateTime <> '' then
    if CheckDate(MDateTime) then RefreshAdministrations;
  //edtDate.text := zDateTime;

end;

procedure TfrmEditMedLogAdminSelect.edtDateKeyPress(Sender: TObject;
  var Key: Char);
begin
  if (key = #13) and (edtDate.Text <> '') then
    if CheckDate(edtDate.Text) then RefreshAdministrations
    else
    begin
      edtDate.Clear;
      edtDate.SetFocus;
    end;
end;

procedure TfrmEditMedLogAdminSelect.edtDateChange(Sender: TObject);
begin
  edtDate.Tag := 0;
end;

end.

