unit fExpiredPatches;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ComCtrls, StdCtrls, BCMA_Objects, BCMA_Util;

procedure ProcessGivenExpiredPatches;

const
  WristBandReasons: array[0..2] of string = (
    'Damaged Wristband',
    'Scanning Equipment Failure',
    'Unable to Determine'
    );
  AdminReasons: array[0..2] of string = (
    'Damaged Medication Label',
    'Scanning Equipment Failure',
    'Unable to Determine'
    );

type
//  EMarkRemovedError = class(Exception)
//  end;

  TfrmExpiredPatches = class(TForm)
    pnlAdminInfo: TPanel;
    pnlUserInput: TPanel;
    pnlButtons: TPanel;
    bvlAdminInfo: TBevel;
    lblAdminInfo: TLabel;
    bvlUnableToScanReason: TBevel;
    btnRemove: TButton;
    btnIgnore: TButton;
    lvwMedications: TListView;
    lblDispensedMeds: TLabel;
    lblMedicationCaption: TLabel;
    lblSchedAdminTimeCaption: TLabel;
    lblScheduleTypeCaption: TLabel;
    lblLastActionCaption: TLabel;
    lblMedication: TLabel;
    lblSchedAdminTime: TLabel;
    lblScheduleType: TLabel;
    lblLastAction: TLabel;
    lblEnterAComment: TLabel;
    memComment: TMemo;
    lbl150CharMax: TLabel;
    imgIcon: TImage;
    lblNotice: TLabel;
    lblDosageCaption: TLabel;
    lblUnitsPerDoseCaption: TLabel;
    lblDosage: TLabel;
    lblUnitsPerDose: TLabel;
    lblMedRouteCaption: TLabel;
    lblMedRoute: TLabel;
    lblOrderStatusCaption: TLabel;
    lblOrderStopDateTimeCaption: TLabel;
    lblOrderStatus: TLabel;
    lblOrderStopDateTime: TLabel;
    Bevel2: TBevel;
    lblPatchNumber: TLabel;
    btnNext: TButton;
    Bevel1: TBevel;
    lvlNotice2: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure btnIgnoreClick(Sender: TObject);
    procedure btnRemoveClick(Sender: TObject);
    procedure btnNextClick(Sender: TObject);
  private
    { Private declarations }
    CurrentMedOrder: TBCMA_MedOrder;
    CurrentInteration: integer;
    procedure PopulateForm;
    Procedure ProcessNextPatch;
  public
    { Public declarations }
  end;

var
  frmExpiredPatches: TfrmExpiredPatches;
  ScannedBagID: string;
implementation

{$R *.dfm}
uses
  BCMA_Common, BCMA_Main;

procedure ProcessGivenExpiredPatches;
var
  x: integer;
  ErrString: String;
begin
  if BCMA_Patient.GivenExpiredPatches.Count = 0 then exit;
  try
    with TfrmExpiredPatches.Create(Application) do
    try
      ProcessNextPatch;
      ShowModal;
    finally
      free;
    end;
  except
    on e:EMarkRemovedError do
      begin
        ErrString := e.Message;
        ErrString := ErrString + #13 + 'The VDL may have been refreshed to reflect' +
        ' the most current information. The process of checking for GIVEN patches on ' +
        'EXPIRED or DC''d orders has been aborted. ' + #13 +  'Please check the order status of all ' +
        'orders on the VDL that have a patch at a GIVEN status';
        DefMessageDlg('Error accessing ', mtError, [mbOK], 0);
       end;
  end;
end;

procedure TfrmExpiredPatches.btnIgnoreClick(Sender: TObject);
var
  CommentText: String;
begin
  btnIgnore.Enabled := False;
  CommentText := StripString(memComment.Text);
  If CommentText <> '' then
  with CurrentMedOrder do
  begin
    AdditionalComments.Add(CommentTExt);
    LogComments(MedLogIEN, AdditionalComments);
  end;
  //ProcessNextPatch;
  btnRemove.Enabled := False;
  btnNext.Enabled := True;
  btnNext.SetFocus;
end;

procedure TfrmExpiredPatches.btnNextClick(Sender: TObject);
begin
  ProcessNextPatch;
end;

procedure TfrmExpiredPatches.btnRemoveClick(Sender: TObject);
begin
  btnRemove.Enabled := False;
  if MemComment.text <> '' then
  CurrentMedOrder.UserComments := StripString(MemComment.text);
  if frmMain.MarkRemoved(CurrentMedOrder) then
  begin
    btnIgnore.Enabled := False;
    btnNext.Enabled := True;
    btnNext.SetFocus;
    //ProcessNextPatch;
  end
  else
    raise EMarkRemovedError.Create('A problem occured during the patch removal process!' + #13 +
      'Please check the administration on the VDL.');
//  btnRemove.Enabled := True;
end;

procedure TfrmExpiredPatches.FormCreate(Sender: TObject);
begin
  CurrentInteration := -1;
end;

procedure TfrmExpiredPatches.PopulateForm;
var
  x: integer;
begin
  with CurrentMedOrder do
  begin
    If OrderedDrugCount = 1 then
      lblUnitsPerDose.Caption := OrderedDrugs[0].QtyOrderedText
    else
      lblUnitsPerDose.Caption := '***Multiple Dispensed Drugs:';

    If (OrderedDrugCount <> 1) or (OrderedDrugs[0].QtyOrdered <> 1) or (pos('.', OrderedDrugs[0].QtyOrderedText) <> 0) then
    begin
      lblUnitsPerDoseCaption.Font.Style := [fsBold];
      lblUnitsPerDose.Font.Style   := [fsBold];
    end;
    lblMedication.Caption := ActiveMedication;
    lblScheduleType.Caption := ScheduleTypeTitles[ScheduleTypeID];
    lblSchedAdmintime.Caption := DisplayVADateYearTime(AdministrationTime);
    lblDosage.Caption := Dosage;
    lblMedRoute.Caption := UpperCase(Route);
    if LastActivityStatus <> '' then
      lblLastAction.Caption := GetLastActivityStatus(LastActivityStatus) + ': ' + DisplayVADateYearTime(TimeLastAction);
    lvwMedications.Clear;
    for x := 0 to OrderedDrugCount - 1 do
      lvwMedications.AddItem(OrderedDrugs[x].Name, nil);

    lblOrderStatus.Caption := GetOrderStatus(OrderStatus);
    lblOrderStopDateTime.caption := DisplayVADateYearTime(StopDateTime);
    imgIcon.Picture.Icon.Handle := LoadIcon(0, IconArray[2]);
    memComment.Clear;
  end;
end;

procedure TfrmExpiredPatches.ProcessNextPatch;
begin
  CurrentInteration := CurrentInteration + 1;
  if CurrentInteration > BCMA_Patient.GivenExpiredPatches.Count - 1 then
  begin
    ModalResult := mrOk;
    exit;
  end;
  CurrentMedOrder := TBCMA_MedOrder(BCMA_Patient.GivenExpiredPatches[CurrentInteration]);
  PopulateForm;
  lblPatchNumber.Caption := 'Patch ' + intToStr(CurrentInteration + 1) + ' of ' +
    IntToStr(BCMA_Patient.GivenExpiredPatches.Count);
  if CurrentInteration = BCMA_Patient.GivenExpiredPatches.Count - 1 then
    btnNext.Caption := '&Done'
  else
    btnNext.Caption := '&Next';

  btnRemove.Enabled := True;
  btnIgnore.Enabled := True;
  if self.Visible then btnIgnore.SetFocus;
  btnNext.Enabled := False;
end;

end.

