unit fPRNMedLog;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, BCMA_Objects, BCMA_Common, BCMA_Util,
  mVitals;

type
  TfrmPRNMedLog = class(TForm)
    pnlData: TPanel;
    pnlButtons: TPanel;
    btnOK: TButton;
    btnCancel: TButton;
    btnMedHistory: TButton;
    lblActiveMed: TLabel;
    lblActiveMedicationName: TLabel;
    lblDispensedDrug: TLabel;
    lblSpecialInstructions: TLabel;
    memSpecialInstructions: TMemo;
    lblLastFourActions: TLabel;
    lvwLastFourActions: TListView;
    lblSelectReason: TLabel;
    cbxReason: TComboBox;
    lblUnitsGivenMessage: TLabel;
    lstDispensedDrugs: TListBox;
    fraVitals1: TfraVitals;
    cbxPainScore: TComboBox;
    lblPainScore: TLabel;
    bvlLine1: TBevel;
    bvlLine2: TBevel;
    GroupBox1: TGroupBox;
    lblScheduleCaption: TLabel;
    imgIcon: TImage;
    lblSchedule: TLabel;
    lblLastActionCaption: TLabel;
    lblLastActionSinceLast: TLabel;
    lblLastActionDateTime: TLabel;
    chkAck: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure cbxReasonChange(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure btnMedHistoryClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cbxPainScoreChange(Sender: TObject);
    procedure cbxReasonKeyPress(Sender: TObject; var Key: Char);
    procedure cbxPainScoreKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
  public
    { Public declarations }
    MedOrder: TBCMA_MedOrder;
  end;

var
  frmPRNMedLog: TfrmPRNMedLog;

implementation

{$R *.dfm}

uses BCMA_main, ReportRequest;

procedure TfrmPRNMedLog.FormShow(Sender: TObject);
var
  x: integer;
begin
  with MedOrder do
  begin
    lblActiveMedicationName.Caption := ActiveMedication;
    memSpecialInstructions.text := SpecialInstructions;

    with BCMA_SiteParameters do
      for x := 0 to ListReasonsGivenPRN.Count - 1 do
        cbxReason.AddItem(piece(ListReasonsGivenPRN[x], '|', 1),
          ptr(StrToInt(piece(ListReasonsGivenPRN[x], '|', 2))));
    //cbxReason.Items.Assign(BCMA_SiteParameters.ListReasonsGivenPRN);

    with cbxPainScore do
    begin
      AddItem('0 - No Pain', ptr(0));
      AddItem('1 - Slightly Uncomfortable', ptr(1));
      for x := 2 to 9 do
        AddItem(IntToStr(x), ptr(x));
      AddItem('10 - Worst Imaginable', ptr(10));
      AddItem('99 - Unable to Respond', ptr(99));
    end;

    if OrderTypeID = otUnitDose then
      lstDispensedDrugs.items.Assign(OrderedDrugNames)
    else
    begin
      for x := 0 to Additives.Count - 1 do
        lstDispensedDrugs.AddItem(piece(Additives[x], '^', 3) + ' ' + piece(Additives[x], '^', 4), nil);
      for x := 0 to Solutions.Count - 1 do
        lstDispensedDrugs.AddItem('  ' + piece(Solutions[x], '^', 3) + ' ' + piece(Solutions[x], '^', 4), nil);
    end;
    for x := 0 to LastFourActions.Count - 1 do
    begin
      lvwLastFourActions.AddItem(DisplayVADateYearTime(piece(LastFourActions.Strings[x], '^', 1)), nil);
      lvwLastFourActions.Items[x].SubItems.Add(piece(LastFourActions.Strings[x], '^', 2));
      lvwLastFourActions.Items[x].SubItems.Add(piece(LastFourActions.Strings[x], '^', 3));
      lvwLastFourActions.Items[x].SubItems.Add(piece(LastFourActions.Strings[x], '^', 4));
      //if piece(LastFourActions.Strings[x], '^' , 7) <> '' then
      //  lvwLastFourActions.Items[x].SubItems.Add(piece(LastFourActions.Strings[x], '^' ,7))
      //else
      lvwLastFourActions.Items[x].SubItems.Add(piece(LastFourActions.Strings[x], '^', 6))

    end;
    imgIcon.Picture.Icon.Handle := LoadIcon(0, IconArray[2]);
    lblSchedule.Caption := Schedule;
    if LastGivenDateTime <> '' then
    begin
      lblLastActionSinceLast.Caption := DaysHoursMinutesPast(LastGivenDateTime) + ' ago on';
      lblLastActionDateTime.Caption := DisplayVADateYearTime(LastGivenDateTime);
    end
    else
    begin
      lblLastActionSinceLast.Caption := '';
      lblLastActionDateTime.Caption := '';
    end;

  end;
end;

procedure TfrmPRNMedLog.cbxReasonChange(Sender: TObject);
begin
  cbxPainScore.ItemIndex := -1;
  if (Integer(cbxReason.Items.Objects[cbxReason.ItemIndex]) = 0) and
    (cbxReason.ItemIndex <> -1) then
  begin
    //btnOK.Enabled := True;
    cbxPainScore.Enabled := False
  end
  else
  begin
    cbxPainScore.Enabled := True;
    //btnOk.Enabled := False;
  end;
end;

procedure TfrmPRNMedLog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if ModalResult = mrCancel then
  begin
    canClose := true;
    exit;
  end;

  if cbxReason.itemIndex = -1 then
  begin
    DefMessageDlg('You must select a reason!', mtInformation, [mbOK], 0);
    CanClose := False;
    exit;
  end;

  if cbxPainScore.Enabled and (cbxPainScore.itemIndex = -1) then
  begin
    DefMessageDlg('You must select a pain score!', mtInformation, [mbOK], 0);
    CanClose := False;
    exit;
  end;

  if not chkAck.Checked then
  begin
    DefMessageDlg('You must check the checkbox in order to complete the documentation of this administration', mtInformation, [mbOK], 0);
    CanClose := False;
    exit;
  end;

//  canClose := (modalResult = mrCancel) or (cbxReason.itemIndex <> -1);
  if canClose = true then
    cmtReasonGivenPRN := cbxReason.Items[cbxReason.itemIndex];
end;

procedure TfrmPRNMedLog.btnMedHistoryClick(Sender: TObject);
begin
  with MedOrder do
    MedicationHistoryReport(PatientIEN, OrderableItemIEN, OrderNumber);
end;

procedure TfrmPRNMedLog.FormCreate(Sender: TObject);
begin
  fraVitals1.LoadVitals;

end;

procedure TfrmPRNMedLog.cbxPainScoreChange(Sender: TObject);
begin
  if cbxPainScore.ItemIndex <> -1 then
    btnOk.Enabled := True;
end;

procedure TfrmPRNMedLog.cbxReasonKeyPress(Sender: TObject; var Key: Char);
begin
  //Prevent a backspace.  Apparently under W2K, a backspace will set
  //itemindex to -1.
  if Key = #8 then
    Key := #0;
end;

procedure TfrmPRNMedLog.cbxPainScoreKeyPress(Sender: TObject;
  var Key: Char);
begin
  //Prevent a backspace.  Apparently under W2K, a backspace will set
  //itemindex to -1.
  if Key = #8 then
    Key := #0;
end;

end.

