unit fPtConfirmation;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, BCMA_Objects, BCMA_Common;

function ConfirmPatient: Boolean;

type
  TfrmPtConfirmation = class(TForm)
    pnlDemographics: TPanel;
    lblSSN: TLabel;
    lblName: TLabel;
    lblWard: TLabel;
    lblRmBd: TLabel;
    pnlButtons: TPanel;
    btnOK: TButton;
    btnCancel: TButton;
    btnContinue: TButton;
    lblDOB: TLabel;
    grpSensitive: TGroupBox;
    grpPatientFlags: TGroupBox;
    memPatientFlags: TMemo;
    grpAllergies: TGroupBox;
    lblAllergies: TLabel;
    memAllergies: TMemo;
    lblADRs: TLabel;
    memADRs: TMemo;
    btnFlagDetails: TButton;
    edtName: TEdit;
    edtSSN: TEdit;
    edtDOB: TEdit;
    edtWard: TEdit;
    edtRmBd: TEdit;
    memSensitiveMSG: TMemo;
    pnlButtonMessage: TPanel;
    pnlMessage: TPanel;
    cbSpeedbump_Patient_Identifiers: TCheckBox;
    procedure cbSpeedbump_Patient_IdentifiersClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnContinueClick(Sender: TObject);
    procedure btnFlagDetailsClick(Sender: TObject);
    procedure pnlButtonMessageEnter(Sender: TObject);
    procedure pnlButtonMessageExit(Sender: TObject);
    procedure pnlMessageEnter(Sender: TObject);
    procedure pnlMessageExit(Sender: TObject);
  private
    { Private declarations }
    function PatientRecordText: String;
  public
    { Public declarations }
  end;

var
  frmPtConfirmation: TfrmPtConfirmation;

implementation

uses BCMA_Main, BCMA_Startup, BCMA_Util;

{$R *.dfm}

procedure TfrmPtConfirmation.FormShow(Sender: TObject);
var
  Status: integer;
  msg: string;
begin

  if grpSensitive.Visible = False then
    height := height - grpSensitive.Height;

  if IsUnableToScan and not (ReadOnly or LimitedAccess or EditMedLog) then begin
    cbSpeedbump_Patient_Identifiers.Visible := True;
    btnOk.Enabled := False;
  end else begin
    cbSpeedbump_Patient_Identifiers.Visible := False;
    if not btnContinue.Visible then
      btnOk.Enabled := True;
  end;
end;

function ConfirmPatient: Boolean;
var
  Status: integer;
  msg: string;
begin
  Result := False;
  frmMain.StatusBar.Panels[0].Text := 'Checking Sensitive Patient...';
  frmMain.StatusBar.Repaint;

  BCMA_Patient.CheckSensitive(Status, msg);
  frmMain.StatusBar.Panels[0].Text := '';
  msg := StringReplace(msg, '*', '', [rfReplaceAll]);
  msg := trim(msg);
  with TfrmPtConfirmation.create(Application) do
  try
    with BCMA_Patient do
    begin
      case Status of
        0, 1:
          begin
            edtSSN.Text:= SSN;
            edtDOB.Text := DOB;
            grpSensitive.Visible := (Status = 1);
            if Status = 1 then memSensitiveMsg.Text := msg;
            pnlButtonMessage.Caption := 'Is this the correct patient?';
            btnContinue.Visible := False;
            btnOk.Enabled := False;
          end;
        2:
          begin
            memSensitiveMsg.text := msg;
            edtSSN.text := '*SENSITIVE*';
            edtDOB.text := '*SENSITIVE*';
            pnlButtonMessage.Caption := 'Click accept if you understand the security issues.';
            btnOk.Enabled := False;

            if IsUnableToScan and not(ReadOnly or LimitedAccess or EditMedLog) then
              btnContinue.Enabled := False;
            

          end;
      else
        begin
          memSensitiveMsg.text := msg;
          edtSSN.text := '*SENSITIVE*';
          pnlButtonMessage.Caption := 'Unable to Continue';
          btnContinue.Visible := False;
          btnOk.Enabled := False;
        end;
      end;
      btnFlagDetails.Enabled := frmMain.actionFlag.Enabled;
      memPatientFlags.Text := PatientRecordText;
      if BCMA_Patient.PatientRecordFlags.Count > 0 then btnFlagDetails.Enabled := True;
      edtName.text := Name;
      edtWard.text := Ward;
      edtRmBd.text := RmBed;
      memAllergies.Text := Allergies;
      memADRs.Text := ADRs;
      pnlMessage.Caption := AdminMessage;
      if AdminMessage = '' then pnlMessage.Enabled := False;
      
      btnFlagDetails.Enabled := BCMA_Patient.PatientRecordFlags.Count > 0;
    end;

    Result := (showModal = mrOK);
  finally
    free;
  end;

end;

procedure TfrmPtConfirmation.cbSpeedbump_Patient_IdentifiersClick(
  Sender: TObject);
var
  Status: integer;
  msg: string;
begin
  BCMA_Patient.CheckSensitive(Status, msg);

  if cbSpeedbump_Patient_Identifiers.Checked then
    if Status = 2 then begin
      btnContinue.Visible := True;
      btnContinue.Enabled := True;
      btnOK.Enabled := False;
    end else
      btnOK.Enabled := True
  else begin
    if Status = 2 then begin
      btnContinue.Enabled := False;
      btnOK.Enabled := False;
    end else
      btnOK.Enabled := False
  end;

end;

procedure TfrmPtConfirmation.btnContinueClick(Sender: TObject);
begin
  with BCMA_Patient do
    if BCMA_Broker <> nil then
      with BCMA_Broker do
        if CallServer('DG SENSITIVE RECORD BULLETIN', [IEN], nil) then
          if Results[0] = '0' then
          begin
            DefMessageDlg('Sensitive record logging failed, this patient can not be opened at this time!', mtError, [mbOK], 0);
            exit;
          end
          else
          begin
            edtSSN.text := SSN;
            edtDOB.text := DOB;
            btnOK.Enabled := True;
            btnContinue.Enabled := False;
            //lblButtonMessage.Caption := 'Is this the correct patient?';
            btnCancel.SetFocus;
          end;
end;

function TfrmPtConfirmation.PatientRecordText: String;
//var aPiece, aPatientRecordText: String;
var x: integer;
begin
  with BCMA_Patient do
    if PatientRecordFlags.Count > 0 then
      for x := 0 to PatientRecordFlags.Count - 1 do
        if x = 0 then Result := PatientRecordFlags[x]
        else Result := Result + ', ' + PatientRecordFlags[x];
{
  if BCMA_Patient.PatientRecordFlags <> '' then
    aPatientRecordText := BCMA_Patient.PatientRecordFlags
  else
    exit;
  Result := Piece(aPatientRecordText, '^', 1);
  aPiece := Result;
  Delete(aPatientRecordText, 1, Length(aPiece) + 1);
  while Length(aPatientRecordText) > 0 do
  begin
    aPiece := Piece(aPatientRecordText, '^', 1);
    Result := Result + ', ' + aPiece;
    Delete(aPatientRecordText, 1, Length(aPiece) + 1);
  end;
}
end;

procedure TfrmPtConfirmation.pnlButtonMessageEnter(Sender: TObject);
begin
  pnlButtonMessage.BevelKind := bkSoft;
end;

procedure TfrmPtConfirmation.pnlButtonMessageExit(Sender: TObject);
begin
  pnlButtonMessage.BevelKind := bkNone;
end;

procedure TfrmPtConfirmation.pnlMessageEnter(Sender: TObject);
begin
    pnlMessage.BevelKind := bkSoft;
end;

procedure TfrmPtConfirmation.pnlMessageExit(Sender: TObject);
begin
    pnlMessage.BevelKind := bkNone;
end;

procedure TfrmPtConfirmation.btnFlagDetailsClick(Sender: TObject);
begin
  frmMain.actionFlag.Execute;
end;

end.

