unit fPtSelect;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, ExtCtrls, StdCtrls, BCMA_Startup, BCMA_Objects,
  BCMA_Util, VHA_Objects;

type

  TPtSelectSortType = (stAscending, stDescending);

  PtSelectColumnTypes = (ctPtName,
    ctPtSSN,
    ctPtDOB,
    ctPtRmBed,
    ctPtWard);

  TfrmPtSelect = class(TForm)
    Panel1: TPanel;
    btnOK: TButton;
    btnCancel: TButton;
    tmrPtSelect: TTimer;
    grpPtName: TGroupBox;
    edtPtName: TEdit;
    grpPtList: TGroupBox;
    pnlPatientList: TPanel;
    lvwPtList: TListView;
    procedure tmrPtSelectTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure lvwPtListSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure btnOKClick(Sender: TObject);
    procedure edtPtNameKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure lvwPtListEnter(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure lvwPtListColumnClick(Sender: TObject; Column: TListColumn);
    procedure FormShow(Sender: TObject);

  private
    { Private declarations }

  public
    { Public declarations }//    property PatientList: TList read FPatientlist;
    procedure LoadPatients(SearchString: string);
    procedure ClearPatients;
    procedure RefreshPtList;
  end;

  TBCMA_Patients = class(TObject)
    (*
      Contains information about a Patient.
    *)
  private
    FRPCBroker: TBCMA_Broker;
    fIEN,
      fName,
      fSex,
      fDOB,
      fSSN,
      fWard,
      fRmBed,
      fDOBforDisplay,
      fSSNforDisplay: string;
  public
    constructor Create(RPCBroker: TBCMA_Broker); virtual;
    destructor Destroy; override;

    procedure ClearPatients;
  end;

var
  frmPtSelect: TfrmPtSelect;
  PatientList: TList;
  PtSelectSortCol: PtSelectColumnTypes;
  PtSelectSortType: TPtSelectSortType;

implementation

{$R *.dfm}

uses
  BCMA_Common, BCMA_Main;

function PtSelectSort(Item1, Item2: pointer): Integer;
var
  str1, str2: string;
begin
  result := 1;

  case PtSelectSortCol of
    ctPtName:
      result := AnsiCompareStr(TBCMA_Patients(Item1).fName,
        TBCMA_Patients(Item2).fName);
    ctPtSSN:
      result := AnsiCompareStr(TBCMA_Patients(Item1).fSSNForDisplay,
        TBCMA_Patients(Item2).fSSNForDisplay);
    ctPtDOB:
      begin
        str1 := format('%s|%s', [TBCMA_Patients(Item1).fDOBforDisplay, TBCMA_Patients(Item1).fName]);
        str2 := format('%s|%s', [TBCMA_Patients(Item2).fDOBforDisplay, TBCMA_Patients(Item2).fName]);
        result := AnsiCompareStr(str1, str2);
      end;
    ctPtRmBed:
      result := AnsiCompareStr(TBCMA_Patients(Item1).fRmBed,
        TBCMA_Patients(Item2).fRmBed);
    ctPtWard:
      begin
        str1 := format('%s|%s', [TBCMA_Patients(Item1).fWard, TBCMA_Patients(Item1).fName]);
        str2 := format('%s|%s', [TBCMA_Patients(Item2).fWard, TBCMA_Patients(Item2).fName]);
        result := AnsiCompareStr(str1, str2);
      end;
  end;
  if PtSelectSortType = stDescending then
  begin
    if result = 1 then
      result := -1
    else if result = -1 then
      result := 1;
  end;
end;

procedure TfrmPtSelect.RefreshPtList;
var
  x: integer;
begin
  lvwPtList.Clear;
  if PatientList.Count > 0 then
    for x := 0 to PatientList.Count - 1 do
      with lvwPtList do
      begin
        AddItem(TBCMA_Patients(PatientList[x]).fName, nil);
        Items[x].SubItems.Add(TBCMA_Patients(PatientList[x]).fSSNforDisplay);
        Items[x].SubItems.Add(TBCMA_Patients(PatientList[x]).fDOBforDisplay);
        Items[x].SubItems.Add(TBCMA_Patients(PatientList[x]).fRmBed);
        Items[x].SubItems.Add(TBCMA_Patients(PatientList[x]).fWard);
        visible := True;
      end;
end;

procedure TfrmPtSelect.tmrPtSelectTimer(Sender: TObject);
var
  MultList: TStringList;
  x: integer;

begin
  tmrPtSelect.Enabled := False;
  MultList := TStringList.Create;
  lvwPtList.Clear;
  with MultList do
  begin
    Add('PTLKUP');
    Add(edtPtName.text);
  end;
  lvwPtList.Repaint;
  frmMain.StatusBar.Panels[0].Text := 'Retrieving Patients...';
  frmMain.StatusBar.Repaint;
  ;
  LoadPatients(edtptName.Text);

  if TBCMA_Patients(PatientList[0]).fIEN = '-1' then
  begin
    lvwPtList.Visible := False;
    pnlPatientList.Caption := TBCMA_Patients(PatientList[0]).fName;
    exit;
  end;
  PatientList.Sort(PtSelectSort);
  RefreshPtList;
  frmMain.StatusBar.Panels[0].Text := '';
end;

procedure TfrmPtSelect.FormCreate(Sender: TObject);
begin
  lvwPtList.Column[0].Width := trunc(lvwPtList.Width * 0.33);
  lvwPtList.Column[1].Width := trunc(lvwPtList.Width * 0.12);
  lvwPtList.Column[2].Width := trunc(lvwPtList.Width * 0.12);
  lvwPtList.Column[3].Width := trunc(lvwPtList.Width * 0.12);
  lvwPtList.Column[4].Width := trunc(lvwPtList.Width * 0.28);
  PatientList := TList.Create;

  PtSelectSortCol := ctPtName;
  PtSelectSortType := stAscending;
end;

procedure TfrmPtSelect.FormActivate(Sender: TObject);
begin
  edtPtName.SetFocus;
end;

procedure TfrmPtSelect.lvwPtListSelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
begin
  btnOK.Enabled := True;
end;

procedure TfrmPtSelect.btnOKClick(Sender: TObject);
begin
  if lvwPtList.ItemIndex = -1 then
    Exit;

  {JK 7/12/2008 CQ #125}
  if ReadOnly or LimitedAccess or EditMedLog then
    BCMA_Patient.ScanPatient(TBCMA_Patients(PatientList[lvwPtList.ItemIndex]).fSSN, 1)
  else
    BCMA_Patient.ScanPatient(TBCMA_Patients(PatientList[lvwPtList.ItemIndex]).fSSN, 0);

  if BCMA_Patient.IEN <> '' then
    ModalResult := mrOK;
end;

procedure TfrmPtSelect.edtPtNameKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  tmrPtSelect.Enabled := False;
  if (Key <> 9) then
    tmrPtSelect.Enabled := Length(edtPtName.Text) >= 2;
end;

procedure TfrmPtSelect.lvwPtListEnter(Sender: TObject);
begin
  with lvwPtList do
    if (ItemIndex = -1) and (items.Count > 0) then lvwPtList.ItemIndex := 0;
end;

procedure TfrmPtSelect.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  PatientList.Free;
end;

procedure TfrmPtSelect.LoadPatients(SearchString: string);
var
  MultList: TStringList;
  x: integer;
begin
  MultList := TStringList.Create;
  with MultList do
  begin
    Add('PTLKUP');
    Add(SearchString);
  end;
  ClearPatients;
  if (BCMA_Broker <> nil) then
    with BCMA_Broker do
      if CallServer('PSB MED LOG LOOKUP', ['~!#null#~!'], MultList) then
      begin
        if (Results.Count = 0) or (Results.Count - 1 <> StrToInt(Results[0])) then
          DefMessageDlg(ErrIncompleteData, mtError, [mbOK], 0)
        else if (piece(Results[1], '^', 1) = '-1') then
          with PatientList do
          begin
            add(TBCMA_Patients.Create(BCMA_Broker));
            with TBCMA_Patients(PatientList[PatientList.Count - 1]) do
            begin
              FIEN := '-1';
              FName := piece(Results[1], '^', 2);
            end;
          end
        else
          with PatientList do
            for x := 1 to Results.Count - 1 do
            begin
              add(TBCMA_Patients.Create(BCMA_Broker));
              with TBCMA_Patients(PatientList[PatientList.Count - 1]) do
              begin
                fIEN := piece(Results[x], '^', 1);
                fName := piece(Results[x], '^', 2);
                fSex := piece(Results[x], '^', 3);
                fDOB := piece(Results[x], '^', 4);
                fSSN := piece(Results[x], '^', 5);
                fWard := piece(Results[x], '^', 6);
                fRmBed := piece(Results[x], '^', 7);
                fDOBforDisplay := piece(Results[x], '^', 10);
                fSSNforDisplay := piece(Results[x], '^', 11);
              end;
            end;
      end;
end;

procedure TBCMA_Patients.ClearPatients;
begin
end;

constructor TBCMA_Patients.Create(RPCBroker: TBCMA_Broker);
begin
  inherited create;
  if RPCBroker <> nil then
    FRPCBroker := RPCBroker;
end;

destructor TBCMA_Patients.Destroy;
begin
  FRPCBroker := nil;
  inherited Destroy;
end;

procedure TfrmPtSelect.ClearPatients;
var
  ii: integer;
begin
  if PatientList <> nil then
    with PatientList do
    begin
      for ii := count - 1 downto 0 do
        TBCMA_Patients(items[ii]).free;
      clear;
    end;
end;

procedure TfrmPtSelect.lvwPtListColumnClick(Sender: TObject;
  Column: TListColumn);
begin
  if PtSelectColumnTypes(Column.Index) = PtSelectSortCol then
    if PtSelectSortType = stAscending then
      PtSelectSortType := stDescending
    else if PtSelectSortType = stDescending then
      PtSelectSortType := stAscending;

  PtSelectSortCol := PtSelectColumnTypes(Column.Index);
  PatientList.Sort(PtSelectSort);
  RefreshPtList;
end;

procedure TfrmPtSelect.FormShow(Sender: TObject);
begin
  edtPtName.SetFocus; 
  ForceForegroundWindow(handle)
end;

end.

