unit oReport;

interface
uses
  classes, Dialogs, SysUtils, Forms, VHA_Objects, BCMA_Objects, Controls;

type
  TBCMA_Report = class(TObject)
  private
    FRPCBroker: TBCMA_Broker;

    FReportType: TReportTypes;

    FPatientIEN,
      FStartTime,
      FStopTime,
      FDLParams,
      FPatientWard,
      FOrderNumber,
      FWard,
      FPatientRoomBed,
      FDLParams2,
      FOrderNumber2,
      FQueueDateTime: String;

      FPSBList: TStringList; {JK 4/11/2008 for Param[13]}

      FPSTSort: String; {JK 4/11/2008 Primary,Secondary,tertiery sort string}
      FTypeReason: String; {JK 4/14/2008 holds MSF Detailed Report Type and Reason params}
      FNurseMasCode: String; {JK 6/4/2008 holds MAS or Nurse code}
      FSiteDivisionCode: String;  {JK 7/11/2008 holds site/division code}
                                                                    

    FReportPatientList,
      FItemList: TStringList;

    FMedTherapyCategoryItemIndex: Integer;
    FMultiPatient: Boolean;


  protected
    procedure SetQueueDateTime(parMDateTime: string);
  public
    constructor Create(RPCBroker: TBCMA_Broker); virtual;
    (*
      Allocates memory, initializes storage variables and saves a pointer
      to a global copy of the TBCMA_Broker object
    *)

    destructor Destroy; override;
    (*
      Deallocates memory and sets FRPCBroker := nil;
    *)
    procedure Clear;
    (*
      Intitalizes internal storage variables.
    *)

    property ReportType: TReportTypes read FReportType write FReportType;
    property PatientIEN: string read FPatientIEN write FPatientIEN;
    property StartTime: string read FStartTime write FStartTime;
    property StopTime: string read FStopTime write FStopTime;
    property DLParams: string read FDLParams write FDLParams;
    property PSTSort: string read FPSTSort write FPSTSort; {JK 4/1//2008}
    property PSBList: TStringList read FPSBList write FPSBList; {JK 4/1//2008}
    property TypeReasonParams: string read FTypeReason write FTypeReason; {JK 4/14/2008}
    property NurseMasCode: string read FNurseMasCode write FNurseMasCode; {JK 4/14/2008}
    property SiteDivisionCode: string read FSiteDivisionCode write FSiteDivisionCode; {JK 4/14/2008}
    property PatientWard: string read FPatientWard write FPatientWard;
    property OrderNumber: string read FOrderNumber write FOrderNumber;
    property OrderNumber2: string read FOrderNumber2 write FOrderNumber2;
    property Ward: string read FWard write FWard;
    property PatientRoomBed: string read FPatientRoomBed write FPatientRoomBed;
    property DLParams2: string read FDLParams2 write FDLParams2;
    property ReportPatientList: TStringList read FReportPatientList write FReportPatientList;
    property ItemList: TStringList read FItemList write FItemList;
    property QueueDateTime: String read FQueueDateTime write SetQueueDateTime;
    property MedTherapyCategoryItemIndex: integer read FMedTherapyCategoryItemIndex write FMedTherapyCategoryItemIndex;
    property MultiPatient: Boolean read FMultiPatient write FMultiPatient;
    procedure Execute(Device: string = '');
    function LoadMedicationList(SearchText, Category: String): Boolean;
    procedure ClearMedicationList;
    procedure ClearSelectedMedications;
    Function BuildMedTherapyMedList: TStringList;
    Function CallReport(Device: String; Count: Integer): integer;

  end;

  {This is meant to be a generic list that can be popopulated with data for the report}
  TBCMA_ReportItemList = class(TObject)
  private
    FItemIEN: string;
    FItemType: string;
    FItemName: string;
  public
    property ItemName: string read FItemName write FItemName;
    property ItemIEN: string read FItemIEN write FItemIEN;
    property ItemType: string read FItemType write FItemType;
  end;

implementation

uses BCMA_Common, BCMA_Startup, BCMA_Util, ReportRequest, Report, BCMA_Main;


procedure TBCMA_Report.ClearMedicationList;
var
  ii: integer;
begin
  if MedicationList <> nil then
    with MedicationList do
    begin
      for ii := count - 1 downto 0 do
        TBCMA_ReportItemList(items[ii]).free;
      clear;
    end;
end;

procedure TBCMA_Report.ClearSelectedMedications;
var
  ii: integer;
begin
  if SelectedMedications <> nil then
    with SelectedMedications do
    begin
      for ii := count - 1 downto 0 do
        TBCMA_ReportItemList(items[ii]).free;
      clear;
    end;
end;

constructor TBCMA_Report.create(RPCBroker: TBCMA_Broker);
begin
  inherited create;

  FReportPatientList := TStringList.Create;
  FItemList := TStringList.Create;
  FPSBList := TStringList.Create; {JK 4/11/2008}
  if RPCBroker <> nil then
    FRPCBroker := RPCBroker;
  clear;
end;

destructor TBCMA_Report.Destroy;
begin
  FReportPatientList.Free;
  FItemList.Free;
  FPSBList.Free; {JK 4/11/2008}
  FRPCBroker := nil;

  inherited Destroy;
end;

function TBCMA_Report.BuildMedTherapyMedList: TStringList;
var
  x: integer;
  ListOut: TStringList;
  tmpString : string;
begin
  Result := TStringList.Create;
  for x := 0 to SelectedMedications.Count - 1 do
    with TBCMA_ReportItemList(SelectedMedications[x]) do
    begin
      SetPiece(tmpString, '^', 1, 'MT');
      SetPiece(tmpString, '^', 2, ItemType);
      SetPiece(tmpString, '^', 3, ItemIEN);
      Result.Add(tmpString);
    end;
end;

function TBCMA_Report.CallReport(Device: String; Count: integer): Integer;
const
  ErrorMsg = 'There appears to be a problem with the report as the data returned from VistA was blank.';
var
  ModalResult: integer;
  StringList: TStringList;
  CallServerResult: Boolean;
begin
  StringList := TStringList.Create;
  //This is to get around broker getting flustered over an empty list
  if ItemList.Count = 0 then
    StringList := nil
  else
    StringList.Assign(ItemList);


  frmMain.StatusBar.Panels[0].Text := 'Running Report...';
  frmMain.StatusBar.Repaint;


  with BCMA_Broker do begin

//  if FPSTSort <> '' then
  if FTypeReason <> '' then
    CallServerResult :=
      CallServer('PSB REPORT',
        [ReportTypeCodes[ReportType],
          PatientIEN,
          FStartTime,
          FStopTime,
          FDLParams,
          Device,
          FPatientWard,
          FOrderNumber,
          FWard,
          FPatientRoomBed,
          FDLParams2,
          FOrderNumber2,
          FQueueDateTime,
          '',
          FPSTSort,
          FTypeReason
          //,FSiteDivisionCode
          //,FNurseMasCode
        ], StringList)
    else
    CallServerResult :=
      CallServer('PSB REPORT',
        [ReportTypeCodes[ReportType],
          PatientIEN,
          FStartTime,
          FStopTime,
          FDLParams,
          Device,
          FPatientWard,
          FOrderNumber,
          FWard,
          FPatientRoomBed,
          FDLParams2,
          FOrderNumber2,
          FQueueDateTime
        ], StringList);

//    if CallServer('PSB REPORT',
//        [ReportTypeCodes[ReportType],
//          PatientIEN,
//          FStartTime,
//          FStopTime,
//          FDLParams,
//          Device,
//          FPatientWard,
//          FOrderNumber,
//          FWard,
//          FPatientRoomBed,
//          FDLParams2,
//          FOrderNumber2,
//          FQueueDateTime,
//          '',
//          FPSTSort
//        ], StringList) then

      if CallServerResult then
      
      begin
        if Results.Count = 0 then
          DefMessageDlg(ErrorMsg, mtError, [mbOk], 0)
        else if StrToIntDef(piece(Results[0], '^', 1), 1) = -1 then
          DefMessageDlg(piece(Results[0], '^', 2), mtError, [mbOk], 0)
        else if Device = '' then
          with TfrmReport.Create(Application) do
          try
            lblCount.Caption := 'Report ' + IntToStr(Count + 1) + ' of ' +
              IntToStr(ReportPatientList.Count);
            ReportText.Text := Results.Text;
            Caption := ReportCaptions[ReportType];
            if REportPatientList.Count > 1 then
            begin
              btnNext.Enabled := True;
              if count = ReportPatientList.count - 1 then
                btnNext.Caption := '&Done';
            end;



            ModalResult := ShowModal;
            //if ModalResult = mrCancel then break;
            Result := ModalResult;
          finally
            free;
            application.ProcessMessages;
          end
        else
          if piece(Results[0], '^', 1) <> '0' then
            DefMessageDlg(piece(Results[0], '^', 2), mtInformation, [mbOk], 0);
      end
      else
        Result := mrAbort;
  end;
  StringList.Free;
end;

procedure TBCMA_Report.Clear;
begin
  //FReportType := ?;
  FPatientIEN := '';
  FStartTime := '';
  FStopTime := '';
  FDLParams := '';
  FDLParams2 := '';
  FPatientWard := '';
  FOrderNumber := '';
  FWard := '';
  FPatientRoomBed := '';
  FOrderNumber2 := '';
  FQueueDateTime := '';
  ItemList.Clear;
  FMedTherapyCategoryItemIndex := 2;
  FPSTSort := '';  {JK 4/11/2008}
end;

procedure TBCMA_Report.Execute(Device: string = '');
var
  x, zResult: integer;
begin
  //if override = true, do use the current logic, rather process the report
  //directly. This will occur if we are currently in the loop (below) and the
  // users selects to print the report from within this loop, thus we'll end up back in the code again
  //before we finished our prior loop.
  if MultiPatient = false then
  begin
    if ReportPatientList.Count = 0 then
      ReportPatientList.Add(PatientIEN);

    for x  := 0 to ReportPatientList.Count - 1 do
    begin
      MultiPatient := True;
      PatientIEN := ReportPatientList[x];
      zResult := CallReport(Device, x);
      if (zResult = mrCancel) or (zResult = mrAbort) then break;
    end;
    ReportPatientList.Clear;
    MultiPatient := False;
  end
  else
    CallReport(Device, -1);

  frmMain.StatusBar.Panels[0].Text := '';
  frmMain.StatusBar.Repaint;

end;

procedure TBCMA_Report.SetQueueDateTime(parMDateTime: string);
begin
  if parMDateTime = '' then
     fQueueDateTime := 'QD^'
  else
    fQueueDateTime := 'QD^' + parMDateTime;
end;
function TBCMA_Report.LoadMedicationList(SearchText, Category: String): Boolean;
var
  x: integer;
begin
  BCMA_Report.ClearMedicationList;
  result := False;
  frmMain.StatusBar.Panels[0].Text := 'Searching for medications...';
  frmMain.StatusBar.Repaint;

  if BCMA_Broker <> nil then
    with BCMA_Broker do
      if CallServer('PSB MOB DRUG LIST', [SearchText, Category], nil) then
        if (Results.Count = 0) or (Results.Count - 1 <> StrToIntDef(Results[0], -1)) then
          DefMessageDlg(ErrIncompleteData, mtError, [mbOK], 0)
        else if piece(Results[1], '^', 1) = '-1' then
          DefMessageDlg(piece(Results[1], '^', 2), mtError, [mbOK], 0)
        else if piece(Results[1], '^', 1) = '-2' then
          DefMessageDlg('Too many results returned!' + #13 + 'Please enter more criteria.', mtInformation, [mbOK], 0)
        else
          with MedicationList do
            for x := 1 to Results.Count - 1 do
            begin
              add(TBCMA_ReportItemList.Create);
              with TBCMA_ReportItemList(MedicationList[MedicationList.count -1]) do
              begin
                FItemType := piece(Results[x], '^', 1);
                FItemIEN := piece(Results[x], '^', 2);
                FItemName := piece(Results[x], '^', 3);
              end;
            end;
  frmMain.StatusBar.Panels[0].Text := '';
  frmMain.StatusBar.Repaint;
end;

end.
