unit InputQry;
{
================================================================================
*	File:  InputQry.PAS
*
*	Application:  Bar Code Medication Administration
*	Revision:     $Revision: 12 $  $Modtime: 2/28/02 5:15p $
*
*	Description:  This is an InputQry form for the application.  This is an
*               InputQuery with enhancements.
*
*	Notes:
*
*
================================================================================
}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Buttons, BCMA_Util;

function inputPrompt(const TitleStr, PromptStr, DefaultStr: string;
  MaxStrLength: Integer; isRequired, isMasked: boolean;
  var CheckState: Boolean; CheckCaption: string): string;
(*
  Uses form TfrmInputQry to prompt for user input.  Input argument DefaultStr
  provides an optional Default value.  If MaxStrLength is greater than zero,
  then length(result) <= MaxStrLength.  If isRequired = True, then a blank
  result is not allowed.  If isMasked = True, then user input is masked.  The
  result is blank, if the user cancels.
*)

type
  TfrmInputQry = class(TForm)
    pnlButton: TPanel;
    lblPrompt: TLabel;
    edtInputQry: TEdit;
    lblMaximum: TLabel;
    btnOK: TButton;
    btnCancel: TButton;
    chkCheckBox: TCheckBox;
    procedure edtInputQryChange(Sender: TObject);
    (*
      Enables the OK button if isBlankAllowed = True or if the edtInputQry
      text value is not blank.
    *)

    procedure btnOKClick(Sender: TObject);
    (*
      Saves the user input into ResultStr and sets the ModalResult = mrOK,
      closing the form.
    *)

    procedure edtInputQryKeyPress(Sender: TObject; var Key: Char);
    {
      If we receive a carriage return (ie, from the scanner), close the form.
    }

    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    {
      Force the form to the from to the foreground as the main form may have focus as a
      result of the shared broker.
    }

  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmInputQry: TfrmInputQry;
  isBlankAllowed: boolean;
  ResultStr: string;

implementation

{$R *.DFM}
uses
  Math;

function inputPrompt(const TitleStr, PromptStr, DefaultStr: string;
  MaxStrLength: Integer; isRequired, isMasked: boolean;
  var CheckState: Boolean; CheckCaption: string): string;
begin
  result := '';
  with TfrmInputQry.Create(application) do
  try
    caption := TitleStr;
    lblPrompt.Caption := PromptStr;
    edtInputQry.Text := DefaultStr;
    edtInputQry.MaxLength := max(0, MaxStrLength);

    with chkCheckBox do
    begin
      Visible := CheckCaption <> '';
      Caption := CheckCaption;
      Checked := CheckState;
    end;
    if edtInputQry.MaxLength > 0 then
      lblMaximum.caption := 'Maximum Length = ' + intToStr(edtInputQry.MaxLength)
    else
      lblMaximum.caption := '';

    isBlankAllowed := not isRequired;
    btnOK.Enabled := not isRequired;

    if isMasked then
      edtInputQry.PasswordChar := '*';

    if (showModal = mrOK) then
      result := ResultStr;
    CheckState := chkCheckBox.Checked;
  finally
    free;
  end;
end;

procedure TfrmInputQry.edtInputQryChange(Sender: TObject);
begin
  btnOK.Enabled := (isBlankAllowed or (edtInputQry.text <> ''));
end;

procedure TfrmInputQry.btnOKClick(Sender: TObject);
begin
  ResultStr := edtInputQry.text;
  modalResult := mrOK;
end;

procedure TfrmInputQry.edtInputQryKeyPress(Sender: TObject; var Key: Char);
begin
  if edtInputQry.text <> '' then
    if key = chr(VK_RETURN) then
    begin
      ResultStr := edtInputQry.text;
      modalResult := mrOK;
    end;

end;

procedure TfrmInputQry.FormActivate(Sender: TObject);
begin
  ForceForegroundWindow(handle);
end;

procedure TfrmInputQry.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

initialization
  isBlankAllowed := True;
end.

